#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>

#ifdef WIN32
#include "dirent.h"
#else
#include <dirent.h>
#endif

#include "d2stats.h"

/* Reads a flat ASCII PvPGN account file (usually stored in users/) and places
   last login IP and last login time into 'acct'. Returns the number of
   successful data extractions from the file. */
unsigned int extract_acct(FILE *file, struct acct_t *acct) {
  char buf[1024], *tok;
  unsigned int pos, ip, part = 0, tokens = 0;
  acct->ip = 0;
  acct->last_time = 0;
  while(fgets(buf, 1024, file) && tokens < 3) {
    if(!strncmp(buf, "\"BNET\\\\acct\\\\lastlogin_ip\"=\"", 28)) {
      buf[strlen(buf) - 1] = '\0';
      ip = 0;
      /* puts the IP into an int. 255.0.15.128 == 0xff000f80 */
      for(pos = 0, tok = strtok(buf + 28, "."); pos < 4 && tok; tok = strtok(NULL, "."), pos++){
	sscanf(tok, "%d", &part);
	ip |= part << (8 * (3 - pos));
      }
      acct->ip = ip;
      tokens++;
    }
    else if(!strncmp(buf, "\"BNET\\\\acct\\\\lastlogin_time\"=\"", 30)) {
      buf[strlen(buf) - 1] = '\0';
      sscanf(buf + 30, "%d", &part);
      acct->last_time = part;
      tokens++;
    }
    else if(!strncmp(buf, "\"BNET\\\\acct\\\\ctime\"=\"", 21)) {
      tokens++; }
  }
  return tokens;
}

/* Using config variable 'acct_path', allocates acct_t entries and reads in
   PvPGN account files one-by-one. Returns the number of accounts added. */
unsigned int read_accts(struct acct_t ***accts) {
  unsigned int acct_no, count;
  DIR *dir;
  FILE *file;
  struct dirent *dp;
  char buf[1024] = "", filename[1024];
  config(acct_path, buf);
  if(buf[0] == '\0' || !(dir = opendir(buf))) {
    return 0; }
  for(count = 0, dp = readdir(dir); dp; dp = readdir(dir)) {
    if(strncmp(dp->d_name, ".", 2) && strncmp(dp->d_name, "..", 3)) {
      count++; }
  }
  closedir(dir);
  if(!(*accts = malloc(count * sizeof(struct acct_t *)))) {
    return 0; }
  dir = opendir(buf);
  for(acct_no = 0, dp = readdir(dir); dp && acct_no < count; dp = readdir(dir)) {
    if(strncmp(dp->d_name, ".", 2) && strncmp(dp->d_name, "..", 3)) {
      strncpy(filename, buf, strlen(buf) + 1);
      strncat(filename, DIR_SLASH, 2);
      strncat(filename, dp->d_name, strlen(dp->d_name) + 1);
      if((file = fopen(filename, "r"))) {
	if(!((*accts)[acct_no] = malloc(sizeof(struct acct_t)))) {
	  return 0; }
	extract_acct(file, (*accts)[acct_no]);
	fclose(file); }
      acct_no++;
    }
  }
  closedir(dir);
  return count;
}

/* If config var 'parse_accts' is set, will extract data from all account files
   and use this data to determine how many total, unique and active accounts
   exist on the server, writing this data to 'list'. */
void parse_accounts(struct list_t *list) {
  unsigned int count, unique = 0, active = 0, acct_no, pos, curr_time, *ips;
  time_t dummy;
  const unsigned int time_lim = 3600 * 24 * 5;
  char buf[1024] = "";
  struct acct_t **accts;
  enum flag_t is_unique;
  config(parse_accts, buf);
  curr_time = (unsigned int)time(&dummy);
  if(!strncmp(buf, "yes", 4)) {
    if(!(count = read_accts(&accts))) {
      write_log("could not open account files\n", 3);
      return; }
    if(!(ips = malloc(count * sizeof(unsigned int)))) {
      return; }
    zero_out(ips, count * sizeof(unsigned int));
    for(acct_no = 0; acct_no < count; acct_no++) {
      /* checks previous ips to see whether the current is unique. */
      is_unique = yes;
      for(pos = 0; pos < count && ips[pos]; pos++) {
	if(ips[pos] == accts[acct_no]->ip) {
	  is_unique = no;
	  break; }
      }
      /* if it is unique, add this to 'ips' so that other accts with the same
         IP later in the array will be marked non-unique. */
      if(is_unique == yes) {
	unique++;
	if(pos < count) {
	  ips[pos] = accts[acct_no]->ip; }
      }
      if(curr_time - time_lim <= accts[acct_no]->last_time) {
	active++; }
    }
    for(acct_no = 0; acct_no < count; acct_no++) {
      free(accts[acct_no]);
    }
    free(accts);
    free(ips);
    list->accts = count;
    list->unique_accts = unique;
    list->active_accts = active;
  }
}
