#ifndef _D2ITEMS_H__
#define _D2ITEMS_H__

#include <stdio.h>
#include "d2stats.h"

/* What 'item section' the item was found in - refer to 'Diablo II 1.09 File
   Format' (google it) for more info on that. Basically, where the item is. */
enum item_section_t {
  unmarked, player, corpse, merc, ig };

/* Where the item can be found on the game screen. */
enum item_location_t {
  unknown_loc, stored, equipped, belt, moved, socketed };

/* Where the item is equipped. */
enum item_worn_t {
  not_worn, head, neck, chest, right_hand, left_hand, right_finger, left_finger, waist, feet, hands, alt_right_hand, alt_left_hand };

/* What type of item it is, for length-calculation purposes only. */
enum item_type_t {
  weapon, armor, charm, tome, body, gold, other };

/* What class of item it is. */
enum item_class_t {
  unknown_c, weapon_c, helm_c, bodyarmor_c, shield_c, otherarm_c, ring_c, amu_c, charm_c, socketable_c, insig_c };

/* Where the item is stored. */
enum item_grid_loc_t {
  not_stored, inventory, cube, stash };

/* The quality of the item - this determines the color of the item name. */
enum quality_t {
  unknown_q, normal_q, low_q, high_q, magic_q, set_q, unique_q, rare_q, crafted_q };

/* Simple struct to hold ear information. */
struct ear_t {
  enum char_class_t ear_class;
  uint8 level;
  char name[16]; };

/* Struct to hold an item property, like 'Cold Resist +20%' */
struct prop_t {
  unsigned int id, skill_id, skill_level, time, charges, max_charges;
  int val, param, val_during_time, val_off_time, max, min, length;
  struct prop_t *next; };

/* Holds a LOT of info about the item, including a lot of flags. See
   items_analysis.c to figure out what most of this stuff means. */
struct item_t {
  unsigned int size, affix_count, id;
  enum quality_t qual;
  uint8 row, col, sockets_used;
  uint32 guid;
  uint8 ilvl, alt_gfx;
  uint16 class_info, charm_affix, affix[6], suid, runeword;
  uint8 subqual, name1, name2;
  uint16 def, dura, quantity;
  uint8 max_dura, sockets;
  struct prop_t *norm_pl, *set_pl[5];
  struct item_t *contained_in, *socketed_items[6];
  enum item_section_t section;
  enum item_location_t location;
  enum item_worn_t worn;
  enum item_grid_loc_t grid_loc;
  struct ear_t ear_info;
  enum item_class_t item_class;
  char tag[5], inscription[16], *name;
  uint16 max_dmg, min_dmg, ac, mod_max_dura, mod_dura, req_str, req_dex;
  uint8 block, req_lvl;
  enum flag_t is_identified, is_disabled, is_socketed, is_illegal, is_ear, is_newbie, is_simple, is_ethereal, is_personalized, has_runeword, has_alt_gfx, has_class_info, has_charm_affix, charm_suff, has_aff[6], has_set_pl[5], dmg_enhanced, ac_enhanced, block_enhanced, dura_enhanced, req_enhanced, indestructible;
  uint8 *bytes; };

/* items.c */
/* Extracts items. */
struct prop_t **find_end_of_prop(struct prop_t **in);
struct prop_t **prop_deep_copy(struct prop_t **dest, struct prop_t *src);
enum flag_t is_equipped(struct item_t *item, enum weapon_set_t ws);
void delete_prop(struct prop_t **prop);
void get_item_tag(const uint8 *bytes, char *tag);
unsigned int get_length_of_proplist(const uint8 *bytes, const unsigned int bytes_size, const unsigned int bits);
unsigned int get_item_length(const uint8 *bytes, const unsigned int bytes_size);
unsigned int get_item_count(const uint8 *bytes, const unsigned int bytes_size, unsigned int *item_count);
unsigned int extract_items(const uint8 *bytes, const unsigned int bytes_size, struct item_t **items);
unsigned int extract_item_section(const uint8 *bytes, const unsigned int bytes_size, struct charinfo_t *charinfo);
void mark_item_section(const unsigned int start, const unsigned int end, enum item_section_t section, struct charinfo_t *charinfo);
unsigned int extract_item_sections(FILE *charfile, struct charinfo_t *charinfo);
void free_items(struct charinfo_t *charinfo);

/* items_analysis.c */
/* Uses item bytes to fill the item struct with useful information. */
enum quality_t determine_qual(const uint8 qual);
void analyze_extended_props(struct item_t *item, const uint8 *set_info, const enum char_class_t char_class);
unsigned int add_prop(const uint8 *bytes, const unsigned int bits, struct prop_t **prop, const unsigned int id);
void compile_props(struct prop_t **prop);
unsigned int read_proplist(const uint8 *bytes, struct prop_t **prop, const unsigned int bits);
void analyze_extended_item(struct item_t *item);
unsigned int block_by_class(const enum char_class_t char_class);
void determine_item_loc(struct item_t *item);
void set_item_flags(struct item_t *item);
void get_ear_info(struct item_t *item);
void get_socket_prop(struct item_t *item);
void scan_sets(struct charinfo_t *charinfo);
void analyze_items(struct charinfo_t *charinfo);

#endif
