#include <math.h>
#include <stdio.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"
#include "print.h"
#include "xml.h"

/* Converts a char_class_t enum to a cstring. */
char *char_class_to_cstr(const enum char_class_t char_class) {
  switch(char_class) {
  case ama: return "amazon";
  case asn: return "assassin";
  case bar: return "barbarian";
  case dru: return "druid";
  case nec: return "necromancer";
  case pal: return "paladin";
  case src: return "sorceress";
  default:  return "unknown";
  }
}

/* Converts a char_class_t enum to a 3-character cstring. */
char *abbr_class_to_cstr(const enum char_class_t char_class) {
  switch(char_class) {
  case ama: return "ama";
  case asn: return "asn";
  case bar: return "bar";
  case dru: return "dru";
  case nec: return "nec";
  case pal: return "pal";
  case src: return "src";
  default:  return "unk";
  }
}

/* Returns a character's title, based on class and hardcore status. */
char *char_title_to_cstr(const enum char_title_t char_title, const enum char_class_t char_class, const enum flag_t is_hardcore) {
  switch(char_title) {
  case sir: return is_hardcore == yes ? "Destroyer" : "Slayer";
  case lord: return is_hardcore == yes ? "Conqueror" : "Champion";
  case baron:
    if(is_hardcore == yes) {
      return "Guardian"; }
    switch(char_class) {
    case bar:
    case dru:
    case nec:
    case pal: return "Patriarch";
    case ama:
    case asn:
    case src:
    default: return "Matriarch";
    }
  default: return "";
  }
}

/* Debug output that was used long before I decided to use XML. You can replace
   xml_print with this and see what happens, my guess is that a little bit of
   the code that this uses is out-of-date. */
void print_info(const struct charinfo_t *charinfo) {
  unsigned int item_no;
  const struct stats_t *stats = &charinfo->stats;
  printf("Character info for \"%s\" at %s:\n\n", charinfo->name, charinfo->filename);
  printf("Class: %s\n", char_class_to_cstr(charinfo->char_class));
  printf("Level: %d\t\t\tExperience: %d\n", stats->level, stats->exp);
  printf("Str:  %3d\t\t\tLife: %5.2f / %5.2f\n", stats->str, stats->hp / 256.0, stats->health / 256.0);
  printf("Dex:  %3d\t\t\tStam: %5.2f / %5.2f\n", stats->dex, stats->sp / 256.0, stats->stam / 256.0);
  printf("Vit:  %3d\t\t\tMana: %5.2f / %5.2f\n", stats->vit, stats->mp / 256.0, stats->mana / 256.0);
  printf("Ener: %3d\n", stats->ener);
  printf("Gold: %d\t\t\tStash: %d\n", stats->gold, stats->gold_stash);
  if(stats->skills_left) {
    printf("Unassigned skills: %d\n", stats->skills_left); }
  if(stats->stats_left) {
    printf("Unassigned stats: %d\n", stats->stats_left); }
  printf("Total items: %d\n", charinfo->item_count);
  for(item_no = 0; item_no < charinfo->item_count; item_no++) {
    if(charinfo->items[item_no]->location == socketed) {
      printf("+-- "); }
    printf("Item %d: ", item_no);
    print_item(charinfo->items[item_no], stats->level, charinfo->set_info);
  }
  print_set_bonuses(charinfo);
}
