#include <stdio.h>
#include <math.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"

/* One huge function that prints a single property to the file handle. Returns
   the actual number of properties that have been traversed. Shims everywhere
   oh god. Tread lightly. */
unsigned int print_prop(FILE *file, struct prop_t **prop_out, const unsigned int clvl) {
  struct property_entry_t *entry;
  char *str, *output = "", *tail = "", aux_output[1024], *skill_name = "(unknown skill)";
  int val, min, max, length;
  unsigned int skill_id;
  struct prop_t *prop;
  prop = *prop_out;
  val = prop->val;
  /* I can't even figure out what the hell this shim does. */
  skill_id = prop->skill_id == 197 ? 225 : prop->skill_id;
  if(!(entry = lookup_prop_by_id(prop->id))) {
    return 0; }
  /* 73 is max durability, 159 and 160 are throw damage, 356 is something
     having to do with whether a quest item corresponds to
     normal/nightmare/hell difficulty, and 92 is level requirement which I
     don't think is in the game anyway. All of these don't need to be
     printed and are ignored. */
  if(prop->id == 73 || prop->id == 159 || prop->id == 160 || prop->id == 356 || prop->id == 92) {
    return 0; }
  /* 140 is extra blood, and while it's not printed in the game, I figured,
     why the hell not print it here? */
  if(prop->id == 140) {
    fprintf(file, "Extra Blood");
    return 1; }
  /* This looks up the entry's string from its string index. The indexes are
     stuff like 'ModStr1a' and need to be translated with any one of three
     string tables. If these aren't loaded correctly, it'll just print
     'unknown' and exit. If there's no string index in the table, it'll
     continue with the name of the property instead of the official string. */
  if(entry->str_pos && entry->str_neg) {
    if(!lookup_string_by_name(entry->str_pos) || !lookup_string_by_name(entry->str_neg)) {
      if(prop->param) {
	fprintf(file, "unknown: %s: %d, %d (%d)", entry->name, val, prop->param, entry->str_val); }
      else {
	fprintf(file, "unknown: %s: %d (%d)", entry->name, val, entry->str_val); }
      return 1;
    }
    str = val > 0 ? lookup_string_by_name(entry->str_pos) : lookup_string_by_name(entry->str_neg);
  }
  else {
    str = entry->name; }
  /* Some min/max pairs need special treatment. */
  switch(prop->id) {
  case 17:
    fprintf(file, "+%d%% Enhanced Damage", (prop->min + prop->max) / 2);
    return 1;
  case 21:
  case 23:
    min = prop->val;
    length = 0;
    /* If they appear in pairs, print both and return two. */
    if(prop->next && ((prop->id == 21 && (prop->next)->id == 22) ||
		      (prop->id == 23 && (prop->next)->id == 24))) {
      prop = prop->next;
      max = prop->val;
      fprintf(file, "Adds %d-%d damage", min, max);
      length += 2;
      /* If there's also a two-handed damage pair attached, return four. */
      if(prop->next && (prop->next)->id == 23 && (prop->next)->next && ((prop->next)->next)->id == 24) {
	length += 2; }
      return length;
    }
    /* If only min or max is present, along with its two-handed counterpart,
       print one of them and return two. */
    else if(prop->next && ((prop->id == 21 && (prop->next)->id == 23) ||
			   (prop->id == 22 && (prop->next)->id == 24))) {
      fprintf(file, "+%d %s", val, str);
      return 2; }
    else {
      fprintf(file, "+%d %s", val, str); }
    return 1;
  case 48:
    fprintf(file, "Adds %d-%d fire damage", prop->min, prop->max);
    return 1;
  case 50:
    fprintf(file, "Adds %d-%d lightning damage", prop->min, prop->max);
    return 1;
  case 52:
    fprintf(file, "Adds %d-%d magic damage", prop->min, prop->max);
    return 1;
  case 54:
    /* Length isn't printed, just like in the game. */
    fprintf(file, "Adds %d-%d cold damage", prop->min, prop->max);
    return 1;
  case 57:
    /* Poison damage is tricky, the game prints it in two different ways,
       and the values are based on the length. More details can be found at
       http://www.xmission.com/~trevin/DiabloIIv1.09_Magic_Properties.shtml */
    if(prop->max == prop->min) {
      fprintf(file, "+%d poison damage over %d seconds", (int)floor(((prop->min + prop->max) / 2.0 * prop->length / 256.0) + 0.5), (int)floor((prop->length / 25.0) + 0.5)); }
    else {
      fprintf(file, "Adds %d-%d poison damage over %d seconds", (int)floor(((prop->min + prop->min) / 2.0 * prop->length / 256.0) + 0.5), (int)floor(((prop->max + prop->max) / 2.0 * prop->length / 256.0) + 0.5), (int)floor((prop->length / 25.0) + 0.5)); }
    return 1;
  case 252:
    /* This wasn't displaying correctly, I can't remember why, so it's been
       hardcoded. */
    if(prop->val == 0) {
      break; }
    fprintf(file, "Repairs %d durability in %d seconds", 1, (int)floor((100.0 / prop->val) + 0.5));
    return 1;
  default:
    break;
  }
  /* Every prop entry has a descfunc val that determines how it should be
     output. I'll give an example of a property with that descfunc for each */
  switch(entry->str_type) {
  /* '+10 to Strength' */
  case 1: output = val > 0 ? "+%d" : "%d"; break;
  /* 'Damage Reduced by 10%' */
  case 2: output = "%d%%"; break;
  /* 'Magic Damage Reduced by 5' */
  case 3: output = "%d"; break;
  /* 'Fire Resist +20%' */
  case 4:
  /* 'Hit Causes Monster to Flee +25%' */
  case 5: output = val > 0 ? "+%d%%" : "%d%%"; break;
  /* '+10 to Dexterity (based on character level)' */
  case 6:
    /* AR bonuses and damage to undead/demons are stored in halves */
    if(prop->id == 224 || prop->id == 245 || prop->id == 246) {
      val *= clvl / 2; }
    /* everything else is in eighths */
    else {
      val *= clvl / 8; }
    output = val > 0 ? "+%d" : "%d";
    tail = "(based on character level)";
    break;
  /* 'Fire Resist 30% (based on character level)' */
  case 7:
    /* AR percent bonuses are stored in halves */
    if(prop->id == 225) {
      val *= clvl / 2; }
    /* everything else is in eighths */
    else {
      val *= clvl / 8; }
    output = "%d%%";
    tail = "(based on character level)";
    break;
  /* 'Heal Stamina Plus +50% (based on character level)' */
  case 8:
    val *= clvl / 8;
    output = val > 0 ? "+%d%%" : "%d%%%";
    tail = "(based on character level)";
    break;
  /* 'Attacker Takes Damage of 5 (based on character level)' */
  case 9:
    val *= clvl / 8;
    output = "%d";
    tail = "(based on character level)";
    break;
  /* 'Repairs 5 durability per second' (no, that's not a typo) */
  case 11:
    if(find_str(str, "%d") == 1) {
      sprintf(aux_output, str, val); }
    break;
  /* 'Freezes target +5' */
  case 12: output = val > 0 ? "+%d" : "%d"; break;
  /* '+3 to Amazon Skill Levels' */
  case 13:
    output = val > 0 ? "+%d" : "%d";
    switch(prop->param) {
    case 0: str = "to Amazon Skill Levels"; break;
    case 1: str = "to Sorceress Skill Levels"; break;
    case 2: str = "to Necromancer Skill Levels"; break;
    case 3: str = "to Paladin Skill Levels"; break;
    case 4: str = "to Barbarian Skill Levels"; break;
    case 5: str = "to Druid Skill Levels"; break;
    case 6: str = "to Assassin Skill Levels"; break;
    default: str = "to (unknown class) Skill Levels"; break;
    }
    break;
  /* '+1 to Bow and Crossbow Skills (Amazon Only)' */
  case 14:
    output = val > 0 ? "+%d" : "%d";
    switch((prop->param & 0x7) + (prop->param >> 3 & 0x1ff) * 3) {
    case 0: str = "to Bow and Crossbow Skills (Amazon Only)"; break;
    case 1: str = "to Passive and Magic Skills (Amazon Only)"; break;
    case 2: str = "to Javelin and Spear Skills (Amazon Only)"; break;
    case 3: str = "to Fire Skills (Sorceress Only)"; break;
    case 4: str = "to Lightning Skills (Sorceress Only)"; break;
    case 5: str = "to Cold Skills (Sorceress Only)"; break;
    case 6: str = "to Curses (Necromancer Only)"; break;
    case 7: str = "to Poison and Bone Skills (Necromancer Only)"; break;
    case 8: str = "to Summoning Skills (Necromancer Only)"; break;
    case 9: str = "to Combat Skills (Paladin Only)"; break;
    case 10: str = "to Offensive Auras (Paladin Only)"; break;
    case 11: str = "to Defensive Auras (Paladin Only)"; break;
    case 12: str = "to Masteries (Barbarian Only)"; break;
    case 13: str = "to Combat Skills (Barbarian Only)"; break;
    case 14: str = "to Warcries (Barbarian Only)"; break;
    case 15: str = "to Summoning Skills (Druid Only)"; break;
    case 16: str = "to Shape Shifting Skills (Druid Only)"; break;
    case 17: str = "to Elemental Skills (Druid Only)"; break;
    case 18: str = "to Traps (Assassin Only)"; break;
    case 19: str = "to Shadow Disciplines (Assassin Only)"; break;
    case 20: str = "to Martial Arts (Assassin Only)"; break;
    default: str = "to (unknown tab) Skills (unknown class Only)"; break;
    }
    break;
  /* '5% chance to cast Level 1 Amplify Damage on striking'*/
  case 15:
    /* these strings are stored in the table with %d and %s tokens.
       so, make sure they're really there first, and then use sprintf
       directly for the desired results after looking up the skill ID. */
    if(find_str(str, "%d") == 2 && find_str(str, "%s") == 1) {
      if(lookup_skill_by_id(skill_id)) {
	skill_name = lookup_skill_by_id(skill_id)->name;
	if(lookup_skill_by_id(skill_id)->str_name && lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name)) {
	  skill_name = lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name); }
      }
      sprintf(aux_output, str, val, prop->skill_level, skill_name);
    }
    break;
  /* Level 1 Meditate Aura When Equipped */
  case 16:
    /* same thing, roughly */
    if(find_str(str, "%d") == 1 && find_str(str, "%s") == 1) {
      if(lookup_skill_by_id(prop->param)) {
	skill_name = lookup_skill_by_id(prop->param)->name;
	if(lookup_skill_by_id(prop->param)->str_name && lookup_string_by_name(lookup_skill_by_id(prop->param)->str_name)) {
	  skill_name = lookup_string_by_name(lookup_skill_by_id(prop->param)->str_name); }
      }
      sprintf(aux_output, str, val, skill_name);
    }
    break;
  /* '+50 Defense (Increases During Daytime)' */
  case 17:
    val = prop->val_off_time;
    output = val > 0 ? "+%d" : "%d";
    switch(prop->time) {
    case 0: tail = "(Increases During Daytime)"; break;
    case 1: tail = "(Increases Near Dusk)"; break;
    case 2: tail = "(Increases During Nighttime)"; break;
    case 3: tail = "(Increases Near Dawn)"; break;
    default: tail = "(Increases at unknown time)"; break;
    }
  /* '+5% bonus to Attack Rating (Increase Near Dusk)' */
  case 18:
    val = prop->val_off_time;
    output = val > 0 ? "+%d%%" : "%d%%%";
    switch(prop->time) {
    case 0: tail = "(Increases During Daytime)"; break;
    case 1: tail = "(Increases Near Dusk)"; break;
    case 2: tail = "(Increases During Nighttime)"; break;
    case 3: tail = "(Increases Near Dawn)"; break;
    default: tail = "(Increases at unknown time)"; break;
    }
    break;
  /* '-5% to Enemy Fire Resistance' (why is this in its own func? no idea) */
  case 20: output = val > 0 ? "+%d%%" : "%d%%"; break;
  /* '+50% to Damage versus Rakanishu' or something ... */
  case 22:
    output = val > 0 ? "+%d%%" : "%d%%";
    /* you don't really expect me to read in monster tables just for the one
       oddball item that might have this property, that's likely not even in
       the Goddamned game, do you */
    tail = "(certain monster types)";
    break;
  /* 'Reanimate as: Dracula' (or something) */
  case 23:
    output = "%d%%";
    tail = "(certain monster)";
    break;
  /* 'Level 1 Teleport (50/50 Charges)' */
  case 24:
    if(lookup_skill_by_id(skill_id)) {
      skill_name = lookup_skill_by_id(skill_id)->name;
      if(lookup_skill_by_id(skill_id)->str_name && lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name)) {
	skill_name = lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name); }
    }
    sprintf(aux_output, "Level %d %s (%d/%d Charges)", prop->skill_level, skill_name, prop->charges, prop->max_charges);
    break;
  /* '+5000 to Blessed Hammer (Paladin Only)' */
  case 27:
    if(lookup_skill_by_id(skill_id)) {
      skill_name = lookup_skill_by_id(skill_id)->name;
      if(lookup_skill_by_id(skill_id)->str_name && lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name)) {
	skill_name = lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name); }
    }
    sprintf(aux_output, "+%d to %s", prop->skill_level, skill_name);
    if(skill_id >= 6 && skill_id < 36) {
      tail = "(Amazon Only)"; }
    else if(skill_id >= 36 && skill_id < 66) {
      tail = "(Sorceress Only)"; }
    else if(skill_id >= 66 && skill_id < 96) {
      tail = "(Necromancer Only)"; }
    else if(skill_id >= 96 && skill_id < 126) {
      tail = "(Paladin Only)"; }
    else if(skill_id >= 126 && skill_id < 156) {
      tail = "(Barbarian Only)"; }
    else if(skill_id >= 221 && skill_id < 251) {
      tail = "(Druid Only)"; }
    else if(skill_id >= 251 && skill_id < 281) {
      tail = "(Assassin Only)"; }
    else {
      tail = "(unknown class Only)"; }
    break;
  /* '+20 to Inferno' (no class restriction, and if the class uses it, they
                       only receive +3 at most) */
  case 28:
    if(lookup_skill_by_id(skill_id)) {
      skill_name = lookup_skill_by_id(skill_id)->name;
      if(lookup_skill_by_id(skill_id)->str_name && lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name)) {
	skill_name = lookup_string_by_name(lookup_skill_by_id(skill_id)->str_name); }
    }
    sprintf(aux_output, "+%d to %s", prop->skill_level, skill_name);
    break;
  default:
    fprintf(file, "unknown: %s (%d)", str, entry->str_val);
    return 1;
  }
  /* Some display functions set their own output instead. */
  if(output[0] == '\0') {
    fprintf(file, "%s", aux_output); }
  /* 188 includes properties like '+3 to Summoning Skills (Necromancer Only)',
     I can't entirely remember why this shim was necessary */
  else if(prop->id == 188) {
    fprintf(file, output, val);
    fprintf(file, " %s", str); }
  /* str_val is another table entry that determines what direction / order 
     to print the value and the string. 0 means 'no value', 1 means 'value
     before string' and 2 means 'value after string'. */
  else if(entry->str_val == 0) {
    fprintf(file, "%s", str); }
  else if(entry->str_val == 1) {
    fprintf(file, output, val);
    fprintf(file, " %s", str); }
  else {
    fprintf(file, "%s ", str);
    fprintf(file, output, val); }
  /* Finally, some display functions set a tail, like '(based on character
     level)', so if necessary print that */
  if(tail[0] != '\0') {
    fprintf(file, " %s", tail); }
  return 1;
}

/* Simple function that uses what's called a 'gnome sort' (exotic!) to
   perform a very, very simple sort on a property list of, like, maybe 10
   properties at most (up to perhaps 50 or so for totals if someone's really
   loaded) by the 'priority' entry in the game table. */
void sort_prop(struct prop_t **list, const unsigned int count) {
  unsigned int pos;
  struct prop_t *tmp;
  pos = 0;
  while(pos < count) {
    if(pos == 0 || lookup_prop_by_id(list[pos - 1]->id)->priority <= lookup_prop_by_id(list[pos]->id)->priority) {
      pos++; }
    else {
      tmp = list[pos];
      list[pos] = list[pos - 1];
      list[--pos] = tmp; }
  }
}

/* Simple debug function to print a proplist, no longer used. */
unsigned int print_prop_list(struct prop_t *prop, const unsigned int clvl, const char *sep) {
  unsigned int count, pos, extra;
  struct prop_t *curr, *list[64];
  if(prop == NULL) {
    return 0; }
  for(curr = prop, count = 0; curr != NULL; curr = curr->next, count++);
  for(curr = prop, pos = 0; curr != NULL; curr = curr->next, pos++) {
    if(!lookup_prop_by_id(curr->id)) {
      return 0; }
    list[pos] = curr;
  }
  sort_prop(list, count);
  for(pos = 0; pos < count; pos++) {
    if((extra = print_prop(stdout, &list[pos], clvl))) {
      if(pos != count - 1) {
	printf("%s", sep); }
      if(extra > 1) {
	pos += extra - 1; }
    }
  }
  return extra ? 1 : 0;
}
