#ifndef _D2_TALLY_H__
#define _D2_TALLY_H__

#include "d2stats.h"
#include "items.h"

/* tally.c */
/* Uses equipped item proplists to tally totals for an item, or for a
   character. */
unsigned int lookup_mana_from_ener(const unsigned int ener, const enum char_class_t char_class);
unsigned int lookup_life_from_vit(const unsigned int vit, const enum char_class_t char_class);
unsigned int lookup_stam_from_vit(const unsigned int vit, const enum char_class_t char_class);
enum flag_t compare_class(const unsigned int c, const enum char_class_t char_class);
unsigned int class_skill_offs(const enum char_class_t char_class);
int get_res_mod(const struct charinfo_t *charinfo);
void free_total(struct prop_t *total);
void get_spec(const struct charinfo_t *charinfo, char *buf);
void add_ear(struct list_t *list, struct ear_t *ear);
enum flag_t add_to_dmg(struct prop_t *prop, double *mult_min, double *mult_max, unsigned int *add_min, unsigned int *add_max);
enum flag_t add_to_def(struct prop_t *prop, double *mult, unsigned int *add);
enum flag_t add_to_block(struct prop_t *prop, unsigned int *add);
enum flag_t get_dmg(struct item_t *item, double *mult_min, double *mult_max, unsigned int *add_min, unsigned int *add_max, const uint8 *set_info);
enum flag_t get_def(struct item_t *item, double *mult, unsigned int *add, const uint8 *set_info);
enum flag_t get_block(struct item_t *item, unsigned int *add, const uint8 *set_info);
enum flag_t get_req(struct item_t *item, double *mult);
enum flag_t get_dura(struct item_t *item, double *mult, unsigned int *add);
double calc_prop_value(struct prop_t *prop);
double calc_item_value(struct item_t *item);
void calc_ac(struct charinfo_t *charinfo);
void calc_block(struct charinfo_t *charinfo);
void calc_ival(struct charinfo_t *charinfo);
void calc_high_set(struct charinfo_t *charinfo);
void link_ears_to_owners(struct list_t *list, struct charinfo_t **characters, const unsigned int character_count);

/* tally_prop.c */
/* Combines all equipped proplists into one, then filters and tallies it
   for extra bonus stats. */
struct prop_t *generate_total_props(const struct charinfo_t *charinfo);
void tally_props(struct prop_t **total, struct charinfo_t *charinfo);

/* tally_list.c */
/* Ranks each character in top-10 lists according to various bonus stats. */
void tally_items(struct charinfo_t *charinfo, struct list_t *list);
void add_to_list(struct charinfo_t **list, struct charinfo_t *entry, const unsigned int list_pos, const unsigned int list_count);
void rank_exp(struct list_t *list, struct charinfo_t *entry);
void rank_ival(struct list_t *list, struct charinfo_t *entry);
void rank_rune(struct list_t *list, struct charinfo_t *entry);
void rank_gold(struct list_t *list, struct charinfo_t *entry);
void rank_mf(struct list_t *list, struct charinfo_t *entry);
void rank_res(struct list_t *list, struct charinfo_t *entry);
void rank_skill(struct list_t *list, struct charinfo_t *entry);
void rank_fcr(struct list_t *list, struct charinfo_t *entry);
void rank_ias(struct list_t *list, struct charinfo_t *entry);
void rank_melee(struct list_t *list, struct charinfo_t *entry);
void rank_alive(struct list_t *list, struct charinfo_t *entry);
void rank_set(struct list_t *list, struct charinfo_t *entry);
void rank_ears_dropped(struct list_t *list, struct charinfo_t *entry);
void test_ear(struct list_t *list);
void rank(struct list_t *list, struct charinfo_t **characters, const unsigned int character_count);

#endif
