#include <stdlib.h>
#include <string.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"
#include "tally.h"

/* Adds to a basic tally in the list if the character is carrying certain
   items, like SoJs and annhi charms */
void tally_items(struct charinfo_t *charinfo, struct list_t *list) {
  unsigned int item_no = 0;
  struct item_t *item;
  for(item_no = 0; item_no < charinfo->item_count; item_no++) {
    item = charinfo->items[item_no];
    if(item->is_ear == yes) {
      add_ear(list, &item->ear_info); }
    else if(!strncmp(item->tag, "std", 4)) { /* Standard of Heroes */
      list->std_count++; }
    else if(item->qual == unique_q) {
      if(item->suid == 122) { /* The Stone of Jordan (ring) */
	list->soj_count++; }
      else if(item->suid == 381) { /* Annhilius (charm) */
	list->annhi_count++; }
      else if(item->suid == 400) { /* Hellfire Torch (charm) */
	list->torch_count++; }
    }
  }
}

/* Adds a character to a certain spot in a list of characters, pushing the
   rest of the characters already in the list down by one. */
void add_to_list(struct charinfo_t **list, struct charinfo_t *entry, const unsigned int list_pos, const unsigned int list_count) {
  unsigned int pos;
  struct charinfo_t *last, *tmp;
  last = list[list_pos];
  list[list_pos] = entry;
  for(pos = list_pos + 1; pos < list_count; pos++) {
    tmp = list[pos];
    list[pos] = last;
    last = tmp;
  }
}

/* The rest of this code is repeated, and self-explanatory. They just determine
   if and where to add characters into a top-20 list of characters with the
   highest of some stat. */
void rank_exp(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 50; pos++) {
    if(!list->by_exp[pos] || (list->by_exp[pos]->stats).exp < (entry->stats).exp) {
      add_to_list(list->by_exp, entry, pos, 50);
      break;
    }
  }
}

void rank_ival(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 20; pos++) {
    if(!list->by_ival[pos] || (list->by_ival[pos]->bonus_stats).ival < (entry->bonus_stats).ival) {
      add_to_list(list->by_ival, entry, pos, 20);
      break;
    }
  }
}

void rank_rune(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 20; pos++) {
    if(!list->by_rune[pos] || (list->by_rune[pos]->bonus_stats).high_rune < (entry->bonus_stats).high_rune || ((list->by_rune[pos]->bonus_stats).high_rune == (entry->bonus_stats).high_rune && (list->by_rune[pos]->bonus_stats).high_rune_count < (entry->bonus_stats).high_rune_count)) {
      add_to_list(list->by_rune, entry, pos, 20);
      break;
    }
  }
}

void rank_gold(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 20; pos++) {
    if(!list->by_gold[pos] || ((list->by_gold[pos]->stats).gold + (list->by_gold[pos]->stats).gold_stash) < ((entry->stats).gold + (entry->stats).gold_stash)) {
      add_to_list(list->by_gold, entry, pos, 20);
      break;
    }
  }
}

void rank_mf(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 20; pos++) {
    if(!list->by_mf[pos] || (list->by_mf[pos]->bonus_stats).mf < (entry->bonus_stats).mf) {
      add_to_list(list->by_mf, entry, pos, 20);
      break;
    }
  }
}

void rank_res(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_res[pos] || ((list->by_res[pos]->bonus_stats).fire + (list->by_res[pos]->bonus_stats).light + (list->by_res[pos]->bonus_stats).cold + (list->by_res[pos]->bonus_stats).pois) < ((entry->bonus_stats).fire + (entry->bonus_stats).light + (entry->bonus_stats).cold + (entry->bonus_stats).pois)) {
      add_to_list(list->by_res, entry, pos, 20);
      break;
    }
  }
}

void rank_skill(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  const unsigned int skills = (entry->bonus_stats).skill0 + (entry->bonus_stats).skill1 / 3 + (entry->bonus_stats).skill2 / 3 + (entry->bonus_stats).skill3 / 3;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_skill[pos] || (list->by_skill[pos]->bonus_stats).skill0 + (list->by_skill[pos]->bonus_stats).skill1 / 3 + (list->by_skill[pos]->bonus_stats).skill2 / 3 + (list->by_skill[pos]->bonus_stats).skill3 / 3 < (int)skills) {
      add_to_list(list->by_skill, entry, pos, 10);
      break;
    }
  }
}

void rank_fcr(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_fcr[pos] || (list->by_fcr[pos]->bonus_stats).fcr < (entry->bonus_stats).fcr) {
      add_to_list(list->by_fcr, entry, pos, 10);
      break;
    }
  }
}

void rank_ias(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_ias[pos] || (list->by_ias[pos]->bonus_stats).ias < (entry->bonus_stats).ias) {
      add_to_list(list->by_ias, entry, pos, 10);
      break;
    }
  }
}

/* This list covers several stats: crushing blow, deadly strike, open wounds */
void rank_melee(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  const unsigned int melee = (entry->bonus_stats).ds * 4 + (entry->bonus_stats).cb * 2 + (entry->bonus_stats).ow;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_melee[pos] || (list->by_melee[pos]->bonus_stats).ds * 4 + (list->by_melee[pos]->bonus_stats).cb * 2 + (list->by_melee[pos]->bonus_stats).ow < (int)melee) {
      add_to_list(list->by_melee, entry, pos, 10);
      break;
    }
  }
}

void rank_alive(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  /* Characters that have died are ineligible for this list */
  if(entry->has_died == yes) {
    return; }
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_alive[pos] || (list->by_alive[pos]->stats).exp < (entry->stats).exp) {
      add_to_list(list->by_alive, entry, pos, 10);
      break;
    }
  }
}

void rank_set(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_set[pos] || (list->by_set[pos]->bonus_stats).set_val < (entry->bonus_stats).set_val || ((list->by_set[pos]->bonus_stats).set_val == (entry->bonus_stats).set_val && count_bits(list->by_set[pos]->set_info[(list->by_set[pos]->bonus_stats).high_set]) < count_bits(entry->set_info[(entry->bonus_stats).high_set]))) {
      add_to_list(list->by_set, entry, pos, 10);
      break;
    }
  }
}

void rank_ears_dropped(struct list_t *list, struct charinfo_t *entry) {
  unsigned int pos;
  for(pos = 0; pos < 10; pos++) {
    if(!list->by_ears_dropped[pos] || (list->by_ears_dropped[pos]->bonus_stats).ears_dropped < (entry->bonus_stats).ears_dropped) {
      add_to_list(list->by_ears_dropped, entry, pos, 10);
      break;
    }
  }
}

/* Ranks all characters based on their bonus stats, and performs a few
   other minor tallies. */
void rank(struct list_t *list, struct charinfo_t **characters, const unsigned int character_count) {
  unsigned int char_no;
  struct charinfo_t *entry;
  enum char_class_t class_no;
  for(char_no = 0; char_no < character_count; char_no++) {
    if(passes_filter(characters[char_no]) == yes) {
      entry = characters[char_no];
      list->char_count++;
      list->class_count[entry->char_class]++;
      list->avg_level += (entry->stats).level;
      list->avg_class_level[entry->char_class] += (entry->stats).level;
      rank_exp(list, entry);
      rank_ival(list, entry);
      rank_rune(list, entry);
      rank_gold(list, entry);
      rank_mf(list, entry);
      rank_res(list, entry);
      rank_skill(list, entry);
      rank_fcr(list, entry);
      rank_ias(list, entry);
      rank_melee(list, entry);
      rank_alive(list, entry);
      rank_set(list, entry);
      rank_ears_dropped(list, entry);
    }
  }
  /* tally the average levels for the server, and for each class */
  if(list->char_count) {
    list->avg_level /= list->char_count;
    for(class_no = ama; class_no <= src; class_no++) {
      if(list->class_count[class_no]) {
	list->avg_class_level[class_no] /= list->class_count[class_no]; }
    }
  }
}
