#ifndef _D2TBL_H__
#define _D2TBL_H__

#include "d2stats.h"
#include "items.h"
#include "hashtable.h"

/* Stores raw data from an ASCII table pulled from the game itself. */
struct table_t {
  unsigned int rows, cols;
  char **header, ***entries; };

/* Item hashtable entry, stores data for various tags pulled from tables. */
struct entry_t {
  enum item_type_t type;
  char *name, *type_tag;
  unsigned int width, height;
  int cost, rarity, req_level, req_str, req_dex, min_dmg, max_dmg, max_sockets, block;
  enum flag_t no_dura, has_quantity, quest, has_def; };

/* Property array entry, stores data pulled from ItemStatCost.txt */
struct property_entry_t {
  char *name, *str_pos, *str_neg;
  unsigned int priority, str_val, bits, bias, param, str_type, encode, add, mult, div; };

/* Unique array entry, stores data pulled from UniqueItems.txt */
struct unique_entry_t {
  char *name, *type_tag;
  unsigned int rarity, req_level, cost_mult, cost_add; };

/* Set item array entry, stores data pulled from SetItems.txt */
struct set_item_entry_t {
  char *name, *set, *type_tag;
  unsigned int rarity, req_level, cost_mult, cost_add; };

/* Skill array entry, stores data pulled from skilldesc.txt */
struct skill_entry_t {
  char *name, *str_name; };

/* Set hardcoded entry, stores hardcoded data compiled into proplists. */
struct set_entry_t {
  char *name;
  unsigned int items, level;  
  struct prop_t *set_bonus[6]; };

/* tbl.c */
/* Handles hashing, reading tables, and most lookup functions. */
unsigned int hash_cstr(void *vstr);
int hash_cmp_cstr(void *l, void *r);
unsigned int read_table(const char *filename, struct table_t *table);
void free_table(struct table_t *table);
void print_col(const struct table_t *table, const int col, const int col2);
struct entry_t *lookup_item_by_tag(char *tag);
struct property_entry_t *lookup_prop_by_id(unsigned int id);
struct skill_entry_t *lookup_skill_by_id(unsigned int id);
struct unique_entry_t *lookup_unique_by_id(unsigned int id);
struct set_item_entry_t *lookup_set_item_by_id(unsigned int id);
char *lookup_magic_pre_by_id(unsigned int id);
char *lookup_magic_suf_by_id(unsigned int id);
char *lookup_rare_pre_by_id(unsigned int id);
char *lookup_rare_suf_by_id(unsigned int id);
char *lookup_rune_by_id(unsigned int id);

/* tbl_string.c */
/* Reads string table data and handles lookup for that data. */
struct hashtable *make_str_hash();
char *lookup_string_by_name(char *tag);

/* tbl_items.c */
/* Reads item entries from item tables. */
char *lookup_item_cols(const struct table_t *table, const unsigned int row, const unsigned int col);
void set_entry_item_type(struct entry_t *entry, const enum item_type_t type);
struct entry_t *get_item_entry(const struct table_t *table, const unsigned int row);
struct hashtable *make_item_hash();

/* tbl_prop.c */
/* Reads prop entries from ItemStatCost.txt */
char *lookup_prop_cols(const struct table_t *table, const unsigned int row, const unsigned int col);
struct property_entry_t *get_prop_entry(const struct table_t *table, const unsigned int row);
struct property_entry_t **make_prop_arr(unsigned int *out_size);
struct skill_entry_t **make_skill_arr(unsigned int *out_size);

/* tbl_affix.c */
/* Reads cstrings from affix table files. */
char **make_magic_pre_arr(unsigned int *out_size);
char **make_magic_suf_arr(unsigned int *out_size);
char **make_rare_pre_arr(unsigned int *out_size);
char **make_rare_suf_arr(unsigned int *out_size);

/* tbl_hardcode.c */
/* Stores lengthy hardcoded magic number tables that correspond to gem and
   rune socket proplists. */
enum item_class_t get_class(struct item_t *item);
void get_gem_prop(struct prop_t **prop_out, const char *tag, const enum item_class_t ic);
void get_rune_prop(struct prop_t **prop_out, const char *tag, const enum item_class_t ic);
void test_hc();

/* tbl_hardcode_sets.c */
/* Generates set bonus proplists from lengthy functions, also handles
   lookup for this data. */
struct prop_t **add_set_prop(struct prop_t **in, const unsigned int id, const int val, const int param, int min, int max, int length, int skill_id, int skill_level, enum flag_t is_res_all);
struct set_entry_t **make_set_arr();
void test_sets();
unsigned int lookup_set_id_by_suid(unsigned int suid);
unsigned int lookup_set_item_id_by_suid(unsigned int suid);
struct set_entry_t *lookup_set_by_suid(unsigned int suid);
struct set_entry_t *lookup_set_by_sid(unsigned int sid);

/* tbl_other.c */
/* Miscellaneous table data reading functions. */
char *lookup_unique_cols(const struct table_t *table, const unsigned int row, const unsigned int col);
char *lookup_set_item_cols(const struct table_t *table, const unsigned int row, const unsigned int col);
struct unique_entry_t *get_unique_entry(const struct table_t *table, const unsigned int row);
struct set_item_entry_t *get_set_item_entry(const struct table_t *table, const unsigned int row);
struct unique_entry_t **make_unique_arr(unsigned int *out_size);
struct set_item_entry_t **make_set_item_arr(unsigned int *out_size);
char **make_rune_arr(unsigned int *out_size);
void rw_test();

#endif
