#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tbl.h"

/* Returns a magic prefix cstring array, containing all the prefix names
   contained in the file specified by config. Sets out_size to its bound. */
char **make_magic_pre_arr(unsigned int *out_size) {
  unsigned int row, keys = 0, vallen, size = 1024;
  char *val, **out, filename[1024];
  struct table_t table;
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(char *)))) {
    return NULL; }
  zero_out(out, size * sizeof(char *));
  config(magic_pre_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, filename);
    return NULL; }
  if(strncmp(table.header[0], "Name", 5)) {
    write_log("table file %s is invalid?\n", 2, filename);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    /* Check to make sure the row is valid (has at least the first two cols),
       then malloc the value cstring (which will contain the name). */
    if(table.entries[row][0][0] != '\0' && table.entries[row][1][0] != '\0') {
      /* If the table has more data than anticipated, resize the array. */
      if(keys > size) {
	size *= 2;
	if(!(out = realloc(out, size * sizeof(char *)))) {
	  free(out);
	  free_table(&table);
	  return NULL;
	}
	zero_out(out + keys, size / 2 * sizeof(char *));
      }
      vallen = (unsigned int)strlen(table.entries[row][0]) + 1;
      if(!(val = malloc(vallen))) {
	free(out);
	free_table(&table);
	return NULL;
      }
      strncpy(val, table.entries[row][0], vallen);
      out[keys] = val;
      keys++;
    }
    /* There are lots of holes in this table, fill them in with nulls. */
    else {
      out[keys] = NULL;
      keys++; }
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}

/* Returns a magic suffix cstring array, containing all the suffix names
   contained in the file specified by config. Sets out_size to its bound. */
char **make_magic_suf_arr(unsigned int *out_size) {
  unsigned int row, keys = 0, vallen, size = 1024;
  char *val, **out, filename[1024];
  struct table_t table;
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(char *)))) {
    return NULL; }
  zero_out(out, size * sizeof(char *));
  config(magic_suf_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, filename);
    return NULL; }
  if(strncmp(table.header[0], "Name", 5)) {
    write_log("table file %s is invalid?\n", 2, filename);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    if(table.entries[row][0][0] != '\0' && table.entries[row][1][0] != '\0') {
      if(keys > size) {
	size *= 2;
	if(!(out = realloc(out, size * sizeof(char *)))) {
	  free(out);
	  free_table(&table);
	  return NULL;
	}
	zero_out(out + keys, size / 2 * sizeof(char *));
      }
      vallen = (unsigned int)strlen(table.entries[row][0]) + 1;
      if(!(val = malloc(vallen))) {
	free(out);
	free_table(&table);
	return NULL;
      }
      strncpy(val, table.entries[row][0], vallen);
      out[keys] = val;
      keys++;
    }
    else {
      out[keys] = NULL;
      keys++; }
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}

/* Returns a rare name1 cstring array, blah blah, see the above functions
   for comments, they're basically the same. */
char **make_rare_pre_arr(unsigned int *out_size) {
  unsigned int row, keys = 0, vallen, size = 512;
  char *val, **out, filename[1024];
  struct table_t table;
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(char *)))) {
    return NULL; }
  zero_out(out, size * sizeof(char *));
  config(rare_pre_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, rare_pre_file);
    return NULL; }
  if(strncmp(table.header[0], "name", 5)) {
    write_log("table file %s is invalid?\n", 2, rare_pre_file);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    if(table.entries[row][0][0] != '\0' && table.entries[row][1][0] != '\0') {
      if(keys > size) {
	size *= 2;
	if(!(out = realloc(out, size * sizeof(char *)))) {
	  free(out);
	  free_table(&table);
	  return NULL;
	}
	zero_out(out + keys, size / 2 * sizeof(char *));
      }
      vallen = (unsigned int)strlen(table.entries[row][0]) + 1;
      if(!(val = malloc(vallen))) {
	free(out);
	free_table(&table);
	return NULL;
      }
      strncpy(val, table.entries[row][0], vallen);
      out[keys] = val;
      keys++;
    }
    else {
      out[keys] = NULL;
      keys++; }
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}

/* Returns a rare name2 cstring array. */
char **make_rare_suf_arr(unsigned int *out_size) {
  unsigned int row, keys = 0, vallen, size = 512;
  char *val, **out, filename[1024];
  struct table_t table;
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(char *)))) {
    return NULL; }
  zero_out(out, size * sizeof(char *));
  config(rare_suf_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, filename);
    return NULL; }
  if(strncmp(table.header[0], "name", 5)) {
    write_log("table file %s is invalid?\n", 2, filename);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    if(table.entries[row][0][0] != '\0' && table.entries[row][1][0] != '\0') {
      if(keys > size) {
	size *= 2;
	if(!(out = realloc(out, size * sizeof(char *)))) {
	  free(out);
	  free_table(&table);
	  return NULL;
	}
	zero_out(out + keys, size / 2 * sizeof(char *));
      }
      vallen = (unsigned int)strlen(table.entries[row][0]) + 1;
      if(!(val = malloc(vallen))) {
	free(out);
	free_table(&table);
	return NULL;
      }
      strncpy(val, table.entries[row][0], vallen);
      out[keys] = val;
      keys++;
    }
    else {
      out[keys] = NULL;
      keys++; }
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}
