#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"
#include "print.h"

/* All values in this file have been pulled from the 'Arreat Summit',
   Blizzard's unofficial Diablo II fansite (http://www.battle.net/diablo2exp/)
   and have been checked and rechecked with debug output functions. */

/* What 'type' entries correspond to a helm. */
const unsigned int sizeof_helm_tags = 4;
const char *helm_tags[] = {"helm", "pelt", "phlm", "circ" };
/* What 'type' entries correspond to a shield. */
const unsigned int sizeof_shield_tags = 3;
const char *shield_tags[] = {"shie", "ashd", "head" };

/* This is a value lookup table for certain properties of gems. */
const unsigned int val1_lookup[] = {
  2, 2, 3, 3, 4,
  4, 8, 12, 16, 20,
  2, 3, 3, 4, 5,
  40, 60, 80, 100, 150,
  8, 12, 18, 24, 30,
  3, 4, 6, 8, 10,
  1, 3, 4, 6, 10,
  12, 16, 22, 28, 40,
  10, 17, 24, 31, 38,
  10, 20, 40, 60, 100,
  12, 16, 22, 28, 40,
  3, 4, 6, 8, 10,
  3, 5, 8, 10, 15,
  12, 16, 22, 28, 40,
  10, 17, 24, 31, 38,
  28, 34, 44, 54, 68,
  6, 8, 11, 14, 19,
  20, 40, 60, 80, 100,
  1, 1, 1, 1, 1,
  12, 16, 22, 28, 40,
  9, 13, 16, 20, 24 };

/* This is a second value lookup table in case the gem has two or more
   properties (a couple do). */
const unsigned int val2_lookup[] = {
  1, 2, 2, 3, 3,
  0, 0, 0, 0, 0,
  8, 8, 12, 12, 19,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  3, 5, 7, 10, 14,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  10, 20, 40, 60, 100,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  4, 8, 12, 16, 20,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  8, 14, 22, 30, 40,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0 };

/* Every gem type has the same property ID types, so this is a lookup
   table for the first property. */
const unsigned int id_lookup[] = {
  60, 78, 74,
  19, 31, 0,
  54, 43, 9,
  57, 45, 2,
  48, 39, 7,
  122, 39, 19,
  50, 41, 80 };

/* This is a lookup table for rune property values. */
const unsigned int rune_val1_lookup[] = {
  1, 1, 1,
  50, 7, 15,
  2, 2, 2,
  0, 30, 30,
  -25, 15, 15,
  9, 15, 15,
  75, 35, 30,
  5, 35, 30,
  1, 35, 30,
  3, 35, 30,
  7, 14, 14,
  9, 7, 7,
  20, 20, 20,
  25, 7, 7,
  -20, -15, -15,
  10, 10, 10,
  10, 10, 10,
  10, 10, 10,
  10, 10, 10,
  75, 50, 50,
  100, 30, 30,
  25, 22, 15,
  0, 7, 7,
  30, 25, 25,
  20, 5, 5,
  7, 5, 5,
  50, 5, 5,
  20, 5, 5,
  1, 50, 5,
  20, 8, 8,
  0, 50, 5,
  3, 0, 0,
  0, 0, 0 };

/* This is a lookup table for rune property IDs. */
const unsigned int rune_id_lookup[] = {
  89, 89, 89,
  124, 20, 154,
  138, 138, 138,
  81, 32, 32,
  116, 27, 27,
  22, 114, 114,
  57, 45, 45,
  48, 39, 39,
  50, 41, 41,
  54, 43, 43,
  60, 78, 78,
  21, 34, 34,
  93, 102, 99,
  112, 74, 74,
  91, 91, 91,
  3, 3, 3,
  1, 1, 1,
  2, 2, 2,
  0, 0, 0,
  79, 79, 79,
  123, 16, 16,
  135, 39, 39,
  117, 35, 35,
  80, 80, 80,
  119, 46, 46,
  62, 40, 40,
  17, 44, 44,
  141, 42, 42,
  113, 9, 77,
  136, 36, 36,
  115, 7, 76,
  134, 153, 153,
  152, 152, 152 };

/* Returns the type of item - weapon, helm, shield, body armor, other armor,
   charm, ring, amulet, charm, socketable, (rune, gem, etc.) insignificant
   (simple or otherwise), or unknown. */
enum item_class_t get_class(struct item_t *item) {
  struct entry_t *entry;
  unsigned int pos;
  if(!(entry = lookup_item_by_tag(item->tag))) {
    return unknown_c; }
  switch(entry->type) {
  case weapon:
    if(!strncmp(entry->type_tag, "tpot", 5)) {
      return insig_c; }
     return weapon_c;
  case armor:
    if(!strncmp(entry->type_tag, "tors", 5)) {
      return bodyarmor_c; }
    for(pos = 0; pos < sizeof_helm_tags; pos++) {
      if(!strncmp(entry->type_tag, helm_tags[pos], 5)) {
	return helm_c; }
    }
    for(pos = 0; pos < sizeof_shield_tags; pos++) {
      if(!strncmp(entry->type_tag, shield_tags[pos], 5)) {
	return shield_c; }
    }
    return otherarm_c;
 case charm:
   return charm_c;
 case tome:
 case body:
 case gold:
   return insig_c;
 case other:
   if(!strncmp(entry->type_tag, "ring", 5)) {
     return ring_c; }
   if(!strncmp(entry->type_tag, "amul", 5)) {
     return amu_c; }
   if(!strncmp(entry->type_tag, "gem", 3)) {
     return socketable_c; }
   if(!strncmp(entry->type_tag, "rune", 5)) {
     return socketable_c; }
   if(!strncmp(entry->type_tag, "jewl", 5)) {
     return socketable_c; }
 default:
   return insig_c;
  }
}

/* Obtains a proplist for a certain gem type, (skull, amethyst, emerald...)
   quality (chipped, normal, flawless, perfect...) and where it's socketed
   into (helm, shield, armor, weapon...) */
void get_gem_prop(struct prop_t **prop_out, const char *tag, const enum item_class_t ic) {
  struct prop_t *prop;
  unsigned int offs;
  if(!(prop = malloc(sizeof(struct prop_t)))) {
    return; }
  zero_out(prop, sizeof(struct prop_t));
  *prop_out = prop;
  if(tag[0] == 's') { /* skulls are tagged differently for some reason */
    switch(tag[2]) {
    case 'c': offs = 0; break; /* chipped */
    case 'f': offs = 1; break; /* flawed */
    case 'u': offs = 2; break; /* normal */
    case 'l': offs = 3; break; /* flawless */
    case 'z': offs = 4; break; /* perfect */
    default: break;
    }
  }
  else {
    switch(tag[1]) {
    case 'c': offs = 0; break; /* chipped */
    case 'f': offs = 1; break; /* flawed */
    case 's': offs = 2; break; /* normal */
    case 'z':
    case 'l': offs = 3; break; /* flawless */ 
    case 'p': offs = 4; break; /* perfect */
    default: break;
    }
  }
  switch(ic) {
  case weapon_c: offs += 5 * 0; break; /* socketed in weapon */
  case shield_c: offs += 5 * 1; break; /* socketed in shield */
  default:       offs += 5 * 2; break; /* socketed in helm / armor */
  }
  if(tag[0] == 's') { /* again, skulls are a special case */
    offs += 15 * 0; }
  else {
    switch(tag[2]) {
    case 'v': offs += 15 * 1; break; /* amethyst (violet) */
    case 'y': offs += 15 * 6; break; /* topaz (yellow) */
    case 'b': offs += 15 * 2; break; /* sapphire (blue) */
    case 'g': offs += 15 * 3; break; /* emerald (green) */
    case 'r': offs += 15 * 4; break; /* ruby (red) */
    case 'w': offs += 15 * 5; break; /* diamond (white) */
    default: break;
    }
  }
  if(offs / 5 == 6) { /* if a sapphire in a weapon, add cold length */
    switch(tag[1]) {
    case 'c': prop->length = (int)(1.0 * 25); break;
    case 'f': prop->length = (int)(1.4 * 25); break;
    case 's': prop->length = (int)(2.0 * 25); break;
    case 'z':
    case 'l': prop->length = (int)(2.4 * 25); break;
    case 'p': prop->length = (int)(3.0 * 25); break;
    default: break;
    }
  }
  if(offs / 5 == 9) { /* if an emerald in a weapon, add poison length */
    switch(tag[1]) {
    case 'c': prop->length = 3 * 25; break;
    case 'f': prop->length = 4 * 25; break;
    case 's': prop->length = 5 * 25; break;
    case 'z':
    case 'l': prop->length = 6 * 25; break;
    case 'p': prop->length = 7 * 25; break;
    default: break;
    }
    /* also, poison min/max needs to be adjusted to match, so just get the id
       and such, assign them here, and return. if it's not a poison dmg ID,
       min/max/length won't matter because they won't be read. */
    prop->val = val1_lookup[offs];
    prop->id = id_lookup[offs / 5];
    prop->min = val1_lookup[offs] * 256 / prop->length;
    prop->max = val2_lookup[offs] * 256 / prop->length;
    return;
  }
  /* lookup table magic */
  prop->val = val1_lookup[offs];
  prop->id = id_lookup[offs / 5];
  /* if socketing a skull into a weapon, also add mana leech / regen */
  if(tag[0] == 's' && ic != shield_c) {
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    /* 62 = mana leech, 27 = mana regen */
    prop->id = ic == weapon_c ? 62 : 27;
    prop->val = val2_lookup[offs];
    return;
  }
  /* if socketing a diamond into a shield, also add every other resist ... */
  else if(tag[2] == 'w' && ic == shield_c) {
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    /* resist lightning */
    prop->id = 41;
    prop->val = val1_lookup[offs];
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    /* resist cold */
    prop->id = 43;
    prop->val = val1_lookup[offs];
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    /* resist poison */
    prop->id = 45;
    prop->val = val1_lookup[offs];
    return;
  }
  /* all other val2 entries are assumed to be part of a min/max pair. */
  else if(val2_lookup[offs]) {
    prop->min = val1_lookup[offs];
    prop->max = val2_lookup[offs]; }
}

/* Obtains a proplist for a certain rune type and where it's socketed into. */ 
void get_rune_prop(struct prop_t **prop_out, const char *tag, const enum item_class_t ic) {
  struct prop_t *prop;
  unsigned int offs, rune_no;
  if(!(prop = malloc(sizeof(struct prop_t)))) {
    return; }
  zero_out(prop, sizeof(struct prop_t));
  *prop_out = prop;
  switch(ic) {
  case weapon_c: offs = 0; break;
  case shield_c: offs = 1; break;
  default:       offs = 2; break;
  }
  /* rune tags are of the form 'rXX', where XX is the rune ID */
  rune_no = tag[2] - '0' + (tag[1] - '0') * 10;
  /* table lookup magic */
  offs += (rune_no - 1) * 3;
  prop->val = rune_val1_lookup[offs];
  prop->id = rune_id_lookup[offs];
  /* the following shims simply add a second property if necessary. */
  if(rune_no == 1) {
    /* El Rune: +50 to light radius, +AR or +armor */
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    /* 19 = AR, 31 = armor */
    prop->id = ic == weapon_c ? 19 : 31;
    prop->val = ic == weapon_c ? 50 : 15;
    return;
  }
  else if(rune_no == 2 && ic == weapon_c) {
    /* Eld Rune: bonus damage to undead if in a weapon */
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    prop->id = 122;
    prop->val = 75;
    return;
  }
  else if(rune_no == 7 && ic == weapon_c) {
    /* Tal Rune: poison damage in a weapon */
    prop->length = 5 * 25;
    prop->min = (int)(prop->val * 256 / prop->length);
    prop->max = (int)(prop->val * 256 / prop->length);
    return;
  }
  else if(rune_no == 8 && ic == weapon_c) {
    /* Ral Rune: fire damage in a weapon */
    prop->min = prop->val;
    prop->max = 30;
    return;
  }
  else if(rune_no == 9 && ic == weapon_c) {
    /* Ort Rune: lightning damage in a weapon */
    prop->min = prop->val;
    prop->max = 50;
    return;
  }
  else if(rune_no == 10 && ic == weapon_c) {
    /* Thul Rune: cold damage in a weapon */
    prop->length = 3 * 25;
    prop->min = prop->val;
    prop->max = 14;
    return;
  }
  else if(rune_no == 21 && ic == weapon_c) {
    /* Pul Rune: 75% extra damage to demons in a weapon */
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    prop->id = 121;
    prop->val = 75;
    return;
  }
  else if(rune_no == 22 && ic != weapon_c) {
    /* Um Rune: All Resistances +15 in helms/armor, +22 in shields */
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    prop->id = 41;
    prop->val = rune_val1_lookup[offs];
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    prop->id = 43;
    prop->val = rune_val1_lookup[offs];
    if(!(prop->next = malloc(sizeof(struct prop_t)))) {
      return; }
    prop = prop->next;
    zero_out(prop, sizeof(struct prop_t));
    prop->id = 45;
    prop->val = rune_val1_lookup[offs];
    return;
  }
  /* Ohm Rune: +50% to min and max dmg */
  else if(rune_no == 27 && ic == weapon_c) {
    prop->min = prop->val;
    prop->max = prop->val;
    return;
  }
}

/* Simple routine to test hardcoded tables above. Not commented. */
void test_hc() {
  char tag[4];
  const char *tags[] = {
    "gcv", "gfv", "gsv", "gzv", "gpv",
    "gcy", "gfy", "gsy", "gly", "gpy",
    "gcb", "gfb", "gsb", "glb", "gpb",
    "gcg", "gfg", "gsg", "glg", "gpg",
    "gcr", "gfr", "gsr", "glr", "gpr",
    "gcw", "gfw", "gsw", "glw", "gpw",
    "skc", "skf", "sku", "skl", "skz"};
  struct prop_t *tmp;
  unsigned int pos;
  tag[0] = 'r';
  tag[3] = '\0';
  for(pos = 1; pos < 34; pos++) {
    tag[1] = pos / 10 + '0';
    tag[2] = pos % 10 + '0';
    printf("(%d) %s, weapon: ", pos, lookup_item_by_tag(tag)->name);
    get_rune_prop(&tmp, tag, weapon_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n(%d) %s, shield: ", pos, lookup_item_by_tag(tag)->name);
    get_rune_prop(&tmp, tag, shield_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n(%d) %s, torso: ", pos, lookup_item_by_tag(tag)->name);
    get_rune_prop(&tmp, tag, bodyarmor_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n");
  }
  for(pos = 0; pos < 35; pos++) {
    tag[0] = tags[pos][0];
    tag[1] = tags[pos][1];
    tag[2] = tags[pos][2];
    printf("(%d) %s, weapon: ", pos, lookup_item_by_tag(tag)->name);
    get_gem_prop(&tmp, tag, weapon_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n(%d) %s, shield: ", pos, lookup_item_by_tag(tag)->name);
    get_gem_prop(&tmp, tag, shield_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n(%d) %s, torso: ", pos, lookup_item_by_tag(tag)->name);
    get_gem_prop(&tmp, tag, bodyarmor_c);
    print_prop_list(tmp, 10, ", ");
    printf("\n");
  }
}
