#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"
#include "print.h"

/* All values in this file have been pulled from the 'Arreat Summit',
   Blizzard's unofficial Diablo II fansite (http://www.battle.net/diablo2exp/)
   and have been checked and rechecked with debug output functions. */

/* Names of each set. */
char *set_strings[] = {
  "Civerb's Vestments",
  "Hsarus' Defense",
  "Cleglaw's Brace",
  "Iratha's Finery",
  "Isenhart's Armory",
  "Vidala's Rig",
  "Milabrega's Regalia",
  "Cathan's Traps",
  "Tancred's Battlegear",
  "Sigon's Complete Steel",
  "Infernal Tools",
  "Berserker's Garb",
  "Death's Disguise",
  "Angelical Raiment",
  "Arctic Gear",
  "Arcanna's Tricks",
  "Natalya's Odium",
  "Aldur's Watchtower",
  "Immortal King",
  "Tal Rasha's Wrappings",
  "Griswold's Legacy",
  "Trang-Oul's Avatar",
  "M'avina's Battle Hymn",
  "The Disciple",
  "Heaven's Brethren",
  "Orphan's Call",
  "Hwanin's Majesty",
  "Sazabi's Grand Tribute",
  "Bul-Kathos' Children",
  "Cow King's Leathers",
  "Naj's Ancient Set",
  "McAuley's Folly" };

/* Amount of items in each set. */
const unsigned int set_sizes[] = {
  3, 3, 3, 4, 4, 4, 4, 5, 5, 6, 3, 3, 3, 4, 4, 4,
  4, 4, 6, 5, 4, 5, 5, 5, 4, 4, 4, 3, 2, 3, 3, 4 };

/* A sort of ilvl for each set. */
const unsigned int set_levels[] = {
  13,  4,  6, 21, 11, 19, 23, 15, 27,  9,  7,  5,  8, 17,  3, 20,
  22, 29, 37, 26, 44, 32, 21, 39, 55, 41, 28, 34, 50, 20, 43, 20 };

/* Adds a property to a set proplist, given the property parameters. */
struct prop_t **add_set_prop(struct prop_t **in, const unsigned int id, const int val, const int param, int min, int max, int length, int skill_id, int skill_level, enum flag_t is_res_all) {
  struct prop_t *curr;
  if(!(curr = malloc(sizeof(struct prop_t)))) {
    return NULL; }
  *in = curr;
  zero_out(curr, sizeof(struct prop_t));
  curr->id = id;
  curr->val = val;
  if(param) {
    curr->param = param; }
  if(max || min) {
    curr->min = min;
    curr->max = max;
    if(length) {
      curr->length = length; }
  }
  if(skill_id || skill_level) {
    curr->skill_id = skill_id;
    curr->skill_level = skill_level; }
  /* if the set prop is '+10 All Resistances', have to add every resist prop */
  if(is_res_all == yes) {
    if(!(curr->next = malloc(sizeof(struct prop_t)))) {
      return NULL; }
    curr = curr->next;
    zero_out(curr, sizeof(struct prop_t));
    curr->id = 41;
    curr->val = val;
    if(!(curr->next = malloc(sizeof(struct prop_t)))) {
      return NULL; }
    curr = curr->next;
    zero_out(curr, sizeof(struct prop_t));
    curr->id = 43;
    curr->val = val;
    if(!(curr->next = malloc(sizeof(struct prop_t)))) {
      return NULL; }
    curr = curr->next;
    zero_out(curr, sizeof(struct prop_t));
    curr->id = 45;
    curr->val = val;
  }
  return &curr->next;
}

/* Returns an array that contains set entries for each set, including their
   bonus proplists. */
struct set_entry_t **make_set_arr() {
  unsigned int set_no, pos;
  struct set_entry_t **out, *entry;
  struct prop_t **curr;
  if(!(out = malloc(sizeof_set * sizeof(struct set_entry_t *)))) {
    return NULL; }
  zero_out(out, sizeof_set * sizeof(struct set_entry_t *));
  for(set_no = 0; set_no < sizeof_set; set_no++) {
    if(!(entry = malloc(sizeof(struct set_entry_t)))) {
      free(out);
      return NULL; }
    zero_out(entry, sizeof(struct set_entry_t));
    entry->name = set_strings[set_no];
    entry->items = set_sizes[set_no];
    entry->level = set_levels[set_no];
    curr = &entry->set_bonus[0];
    /* Commenting this would be way too much effort. You can see where these
       bonus properties come from at the 'Arreat Summit', look up the prop
       IDs in 'ItemStatCost.txt', or you could just run the debug output
       function and confirm that I'm right. bonus[0] is the set bonus you'd
       get by wearing two pieces, bonus[1] is the bonus for wearing three,
       and so on until bonus[4] which is the bonus for completing the set.
       The set IDs are pulled from a file, 'sets.txt' I think is the name. */
    switch(set_no) {
    case 0: /* Civerb's */
      curr = add_set_prop(curr, 39, 15, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 122, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 0, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 41, 25, 0, 0, 0, 0, 0, 0, no);
      break;
    case 1: /* Hsarus' */
      curr = add_set_prop(curr, 78, 5, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 153, 0, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 41, 25, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 22, 5, 0, 0, 0, 0, 0, 0, no);
      break;
    case 2: /* Cleglaw's */
      curr = add_set_prop(curr, 31, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 31, 50, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 136, 35, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 6, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 93, 20, 0, 0, 0, 0, 0, 0, no);
      break;
    case 3: /* Iratha's */
      curr = add_set_prop(curr, 31, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 96, 20, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 2, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 40, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 42, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 44, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 46, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 20, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 4: /* Isenhart's */
      curr = add_set_prop(curr, 0, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 2, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 96, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 20, 30, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 119, 35, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 60, 5, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 10, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 5: /* Vidala's */
      curr = add_set_prop(curr, 19, 75, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 2, 15, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 0, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 54, 15, 0, 15, 20, 2 * 25, 0, 0, no);
      curr = add_set_prop(curr, 134, 1, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 156, 50, 0, 0, 0, 0, 0, 0, no);
      break;
    case 6: /* Milebrega's */
      curr = add_set_prop(curr, 19, 75, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 19, 125, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 83, 2, 6, 0, 0, 0, 0, 2, no);
      curr = add_set_prop(curr, 60, 8, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 45, 15, 0, 0, 0, 0, 0, 0, no);
      break;
    case 7: /* Cathan's */
      curr = add_set_prop(curr, 48, 15, 0, 15, 20, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 41, 25, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 105, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 35, 3, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 25, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 19, 60, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 9, 20, 0, 0, 0, 0, 0, 0, no);
      break;
    case 8: /* Tancred's */
      curr = add_set_prop(curr, 50, 15, 0, 15, 15, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 60, 5, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 62, 5, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 10, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 150, 35, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 79, 75, 0, 0, 0, 0, 0, 0, no);
      break;
    case 9: /* Sigon's */
      curr = add_set_prop(curr, 60, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 31, 100, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 9, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 12, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 49, 24, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 78, 12, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 34, 7, 0, 0, 0, 0, 0, 0, no);
      break;
    case 10: /* Infernal */
      curr = add_set_prop(curr, 57, (int)(8 * 256 / (3 * 25)), 0, (int)(8 * 256 / (3 * 25)), (int)(8 * 256 / (3 * 25)), 3 * 25, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 135, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 83, 1, 2, 0, 0, 0, 0, 1, no);
      curr = add_set_prop(curr, 119, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 6, 0, 0, 0, 0, 0, 0, no);
      break;
    case 11: /* Berserker's */
      curr = add_set_prop(curr, 7, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 31, 75, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 110, 75, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 57, (int)(5 * 256 / (3 * 25)), 0, (int)(5 * 256 / (3 * 25)), (int)(9 * 256 / (3 * 25)), 3 * 25, 0, 0, no);
      break;
    case 12: /* Death's */
      curr = add_set_prop(curr, 60, 8, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 119, 40, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 25, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 21, 10, 0, 0, 0, 0, 0, 0, no);
      break;
    case 13: /* Angelical */
      curr = add_set_prop(curr, 2, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 9, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 118, 0, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 25, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 80, 40, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 27, 8, 0, 0, 0, 0, 0, 0, no);
      break;
    case 14: /* Arctic */
      curr = add_set_prop(curr, 0, 5, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 7, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 153, 0, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 54, 6, 0, 6, 14, 4 * 25, 0, 0, no);
      break;
    case 15: /* Arcanna's */
      curr = add_set_prop(curr, 9, 25, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 7, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 105, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 5, 0, 0, 0, 0, 0, 0, no);
      break;
    case 16: /* Natalya's */
      curr = add_set_prop(curr, 35, 15, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 31, 200, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 83, 3, 3, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 31, 150, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 60, 14, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 14, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 36, 30, 0, 0, 0, 0, 0, 0, no);
      break;
    case 17: /* Aldur's */
      curr = add_set_prop(curr, 119, 150, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 80, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 83, 3, 5, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 17, 350, 0, 350, 350, 0, 0, 0, no);
      curr = add_set_prop(curr, 62, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 31, 150, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 9, 150, 0, 0, 0, 0, 0, 0, no);
      break;
    case 18: /* Immortal King */
      curr = add_set_prop(curr, 19, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 19, 75, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[2];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 19, 125, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[3];
      curr = prop_deep_copy(curr, entry->set_bonus[2]);
      curr = add_set_prop(curr, 19, 200, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[3]);
      curr = add_set_prop(curr, 83, 3, 1, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 7, 150, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 35, 10, 0, 0, 0, 0, 0, 0, no);
      break;
    case 19: /* Tal Rasha's */
      curr = add_set_prop(curr, 74, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 80, 65, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[2];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 99, 25, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[2]);
      curr = add_set_prop(curr, 83, 3, 4, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 7, 150, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 32, 50, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 31, 150, 0, 0, 0, 0, 0, 0, no);
      break;
    case 20: /* Griswold's */
      curr = add_set_prop(curr, 0, 20, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 2, 30, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 83, 3, 6, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 99, 30, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 19, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 7, 150, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 21: /* Trang-Oul's */
      curr = add_set_prop(curr, 27, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 97, 18, 0, 0, 0, 0, 47, 18, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 27, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 97, 13, 0, 0, 0, 0, 51, 13, no);
      curr = &entry->set_bonus[2];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 27, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 97, 10, 0, 0, 0, 0, 56, 10, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[2]);
      curr = add_set_prop(curr, 83, 3, 2, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 60, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 97, 3, 0, 0, 0, 0, 61, 3, no);
      curr = add_set_prop(curr, 31, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 9, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 74, 5, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 27, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 22: /* M'avina's */
      curr = add_set_prop(curr, 0, 20, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 2, 30, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 83, 3, 0, 0, 0, 0, 0, 3, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 31, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 19, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 80, 100, 0, 0, 0, 0, 0, 0, no);
      break;
    case 23: /* Disciple */
      curr = add_set_prop(curr, 31, 150, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 57, (int)(22 * 256 / (3 * 25)), 0, (int)(22 * 256 / (3 * 25)), (int)(22 * 256 / (3 * 25)), 3 * 25, 0, 0, no);
      curr = &entry->set_bonus[2];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 0, 10, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[2]);
      curr = add_set_prop(curr, 127, 2, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 9, 100, 0, 0, 0, 0, 0, 0, no);
      break;
    case 24: /* Heaven's */
      curr = add_set_prop(curr, 28, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 74, 20, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 127, 2, 0, 0, 0, 0, 0, 2, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 153, 0, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 89, 5, 0, 0, 0, 0, 0, 0, no);
      break;
    case 25: /* Orphan's */
      curr = add_set_prop(curr, 7, 35, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 78, 5, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 7, 50, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 15, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 31, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 2, 10, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 0, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 80, 80, 0, 0, 0, 0, 0, 0, no);
      break;
    case 26: /* Hwanin's */
      curr = add_set_prop(curr, 31, 100, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 31, 200, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 127, 2, 0, 0, 0, 0, 0, 2, no);
      curr = add_set_prop(curr, 96, 30, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 60, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 30, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 27: /* Sazabi's */
      curr = add_set_prop(curr, 96, 40, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 60, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 76, 27, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 30, 0, 0, 0, 0, 0, 0, yes);
      break;
    case 28: /* Bul-Kathos' */
      curr = &entry->set_bonus[4];
      curr = add_set_prop(curr, 127, 2, 0, 0, 0, 0, 0, 2, no);
      curr = add_set_prop(curr, 19, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 121, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 122, 200, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 48, 20, 0, 20, 20, 0, 0, 0, no);
      curr = add_set_prop(curr, 31, 25, 0, 0, 0, 0, 0, 0, no);
      break;
    case 29: /* Cow King's */
      curr = add_set_prop(curr, 45, 25, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 201, 25, 0, 0, 0, 0, 42, 5, no);
      curr = add_set_prop(curr, 11, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 0, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 93, 30, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 80, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 79, 100, 0, 0, 0, 0, 0, 0, no);
      break;
    case 30: /* Naj's */
      curr = add_set_prop(curr, 31, 175, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 2, 15, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 39, 50, 0, 0, 0, 0, 0, 0, yes);
      curr = add_set_prop(curr, 0, 20, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 9, 100, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 127, 1, 0, 0, 0, 0, 0, 1, no);
      curr = add_set_prop(curr, 74, 10, 0, 0, 0, 0, 0, 0, no);
      break;
    case 31: /* McAuley's / Sander's */
      curr = add_set_prop(curr, 31, 50, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[1];
      curr = prop_deep_copy(curr, entry->set_bonus[0]);
      curr = add_set_prop(curr, 19, 75, 0, 0, 0, 0, 0, 0, no);
      curr = &entry->set_bonus[4];
      curr = prop_deep_copy(curr, entry->set_bonus[1]);
      curr = add_set_prop(curr, 127, 1, 0, 0, 0, 0, 0, 1, no);
      curr = add_set_prop(curr, 60, 4, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 9, 50, 0, 0, 0, 0, 0, 0, no);
      curr = add_set_prop(curr, 80, 50, 0, 0, 0, 0, 0, 0, no);
      break;
    }
    out[set_no] = entry;
  }
  for(set_no = 0; set_no < sizeof_set; set_no++) {
    for(pos = 0; pos < 5; pos++) {
      compile_props(&out[set_no]->set_bonus[pos]);
    }
  }
  return out;
}

/* Debug output routine for set entries, not commented. */
void test_sets() {
  struct set_entry_t *set, **lookup;
  unsigned int set_no, pos, item_no;
  lookup = make_set_arr();
  for(set_no = 0; set_no < sizeof_set; set_no++) {
    set = lookup[set_no];
    printf("(%d) \"%s\", size %d, level %d\n", set_no, lookup_string_by_name(set->name), set->items, set->level);
    for(pos = 0; pos < 5; pos++) {
      if(set->set_bonus[pos]) {
	if(pos == 4) {
	  printf("complete: "); }
	else {
	  printf("%d items: ", pos + 2); }
	print_prop_list(set->set_bonus[pos], 5, ", ");
	printf("\n");
      }
    }
  }
  for(item_no = 0; item_no < 500; item_no++) {
    if(lookup_set_item_by_id(item_no)) {
      printf("%d (%d, %d): %s ", item_no, lookup_set_id_by_suid(item_no), lookup_set_item_id_by_suid(item_no), lookup_string_by_name(lookup_set_item_by_id(item_no)->name));
      if(lookup_set_by_suid(item_no)) {
	printf("- %s\n", lookup_string_by_name(lookup_set_by_suid(item_no)->name)); }
      else {
	printf("(unknown)\n"); }
    }
  }

}

/* Looks up the set ID of a set item, used to look up its set entry.
   files used: see above
   sample entry: 4 -> 1 (set 1 == 'Hsarus' Defense' */
unsigned int lookup_set_id_by_suid(unsigned int suid) {
  unsigned int set_id, count = 0;
  for(set_id = 0; set_id < sizeof_set; set_id++) {
    if(suid >= count && suid < count + set_sizes[set_id]) {
      return set_id; }
    count += set_sizes[set_id];
  }
  return sizeof_set;
}

/* Looks up the index of a set item within its set, used to calculate which
   set items have been worn yet (mostly to keep dual angelical rings from
   counting as two set items for bonus purposes, really) and other things.
   files used: see above
   sample entry: 4 -> 1 (it's set item two of three */
unsigned int lookup_set_item_id_by_suid(unsigned int suid) {
  unsigned int set_id, count = 0;
  for(set_id = 0; set_id < sizeof_set; set_id++) {
    if(suid >= count && suid < count + set_sizes[set_id]) {
      return suid - count; }
    count += set_sizes[set_id];
  }
  return 1024;
}

/* Looks up a set entry by one of its items' suids, found in the JM format.
   files used: see above
   sample entry: 4 -> 'Hsarus' Defense' */
struct set_entry_t *lookup_set_by_suid(unsigned int suid) {
  static struct set_entry_t **lookup = NULL;
  if(!lookup) {
    if(!(lookup = make_set_arr())) {
      write_log("failed to obtain set array (this shouldn't happen!)\n", 1);
      lookup = (struct set_entry_t **)1;
      return NULL;
    }
  }
  if(lookup == (struct set_entry_t **)1) {
    return NULL; }
  else {
    suid = lookup_set_id_by_suid(suid);
    if(suid > sizeof_set) {
      return NULL; }
    return lookup[suid];
  }
}

/* Looks up a set entry by its set ID index, found in sets.txt.
   files used: see above
   sample entry: 4 -> 'Isenhart's Armory' */
struct set_entry_t *lookup_set_by_sid(unsigned int sid) {
  unsigned int suid = 0, set_no;
  for(set_no = 0; set_no < sid; set_no++) {
    suid += set_sizes[set_no];
  }
  return lookup_set_by_suid(suid);
}
