#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tbl.h"

/* ItemStatCost.txt column list. */
#define sizeof_prop_column_list 14
const char *prop_column_list[] = {
  "ID", "Stat", "descstrpos", "descstrneg", "descpriority", "descval", "descfunc", "Save Bits", "Save Add", "Save Param Bits", "Encode", "Add", "Multiply", "Divide" };

/* Static lookup function that determines which column numbers match which
   column names above. Returns the entry for ('row', 'prop_column_list[col]'),
   or NULL if the column hasn't been found. */
char *lookup_prop_cols(const struct table_t *table, const unsigned int row, const unsigned int col) {
  static unsigned int col_nums[sizeof_prop_column_list], set = 0;
  unsigned int col_find, pos;
  if(table) {
    if(!set) {
      for(pos = 0; pos < table->cols; pos++) {
	for(col_find = 0; col_find < sizeof_prop_column_list; col_find++) {
	  if(!strncmp(table->header[pos], prop_column_list[col_find], strlen(prop_column_list[col_find]) + 1)) {
	    col_nums[col_find] = pos; }
	}
      }
      set = 1;
    }
    if(col_nums[col] == -1) {
      return NULL; }
    return table->entries[row][col_nums[col]];
  }
  else {
    for(col_find = 0; col_find < sizeof_prop_column_list; col_find++) {
      col_nums[col_find] = -1;
    }
    set = 0;
    return NULL;
  }
}

/* Searches a table's row and mallocs a prop entry to match the row. */
struct property_entry_t *get_prop_entry(const struct table_t *table, const unsigned int row) {
  struct property_entry_t *entry;
  char *str, *newstr;
  if(!(entry = malloc(sizeof(struct property_entry_t)))) {
    return NULL; }
  zero_out(entry, sizeof(struct property_entry_t));
  if(!(str = lookup_prop_cols(table, row, 1)) || str[0] == '\0') {
    return NULL; }
  if(!(newstr = malloc(strlen(str) + 1))) {
    return NULL; }
  strncpy(newstr, str, strlen(str) + 1);
  entry->name = newstr;
  if((str = lookup_prop_cols(table, row, 2)) && str[0] != '\0') {
    if(!(newstr = malloc(strlen(str) + 1))) {
      return NULL; }
    strncpy(newstr, str, strlen(str) + 1);
    entry->str_pos = newstr;
  }
  if((str = lookup_prop_cols(table, row, 3)) && str[0] != '\0') {
    if(!(newstr = malloc(strlen(str) + 1))) {
      return NULL; }
    strncpy(newstr, str, strlen(str) + 1);
    entry->str_neg = newstr;
  }
  if((str = lookup_prop_cols(table, row, 4)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->priority); }
  if((str = lookup_prop_cols(table, row, 5)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->str_val); }
  if((str = lookup_prop_cols(table, row, 6)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->str_type); }
  if((str = lookup_prop_cols(table, row, 7)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->bits); }
  if((str = lookup_prop_cols(table, row, 8)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->bias); }
  if((str = lookup_prop_cols(table, row, 9)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->param); }
  if((str = lookup_prop_cols(table, row, 10)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->encode); }
  if((str = lookup_prop_cols(table, row, 11)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->add); }
  if((str = lookup_prop_cols(table, row, 12)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->mult); }
  if((str = lookup_prop_cols(table, row, 13)) && str[0] != '\0') {
    sscanf(str, "%u", &entry->div); }
  return entry;
}

/* Reads ItemStatCost.txt, makes lots of prop entries and stores
   each into an array, which is returned. */
struct property_entry_t **make_prop_arr(unsigned int *out_size) {
  unsigned int row, key = 0, keys = 0, size = 512;
  char *tmp, filename[1024];
  struct property_entry_t *val, **out;
  struct table_t table;
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(struct property_entry_t *)))) {
    return NULL; }
  zero_out(out, size * sizeof(struct property_entry_t *));
  lookup_prop_cols(NULL, 0, 0);
  config(property_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, filename);
    return NULL; }
  if(lookup_prop_cols(&table, 0, 0) == NULL || lookup_prop_cols(&table, 0, 1) == NULL) {
    write_log("table file %s is invalid?\n", 2, filename);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    if((tmp = lookup_prop_cols(&table, row, 0))[0] != '\0') {
      sscanf(tmp, "%u", &key);
      if(!(val = get_prop_entry(&table, row))) {
	return NULL; }
      if(key > size) {
	size *= 2;
	if(!(out = realloc(out, size * sizeof(struct property_entry_t *)))) {
	  free(out);
	  free_table(&table);
	  return NULL;
	}
	zero_out(out + key, size / 2 * sizeof(struct property_entry_t *));
      }
      out[key] = val;
      keys++;
    }
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}

/* Returns a skill name cstring array, containing all the skill names
   contained in the file specified by config. Sets out_size to its bound. */
struct skill_entry_t **make_skill_arr(unsigned int *out_size) {
  unsigned int row, keys = 0, size = 512, skill_desc_col = -1, skill_name_col = -1, col, desc_len, name_len;
  struct skill_entry_t *val, **out;
  struct table_t table;
  char filename[1024];
  zero_out(&table, sizeof(struct table_t));
  if(!(out = malloc(size * sizeof(struct skill_entry_t *)))) {
    return NULL; }
  zero_out(out, size * sizeof(struct skill_entry_t *));
  config(skill_file, filename);
  if(!(read_table(filename, &table))) {
    write_log("could not open table file %s\n", 2, filename);
    return NULL; }
  for(col = 0; col < table.cols; col++) {
    if(!strncmp(table.header[col], "skilldesc", 10)) {
      skill_desc_col = col; }
    else if(!strncmp(table.header[col], "str name", 9)) {
      skill_name_col = col; }
  }
  if(skill_desc_col == -1 || skill_name_col == -1) {
    write_log("table file %s is invalid?\n", 2, filename);
    free_table(&table);
    return NULL; }
  for(row = 0; row < table.rows; row++) {
    if(!(val = malloc(sizeof(struct skill_entry_t)))) {
      free(out);
      free_table(&table);
      return NULL; }
    desc_len = (unsigned int)strlen(table.entries[row][skill_desc_col]) + 1;
    if(!(val->name = malloc(desc_len))) {
      free(out);
      free_table(&table);
      return NULL; }
    strncpy(val->name, table.entries[row][skill_desc_col], desc_len);
    name_len = (unsigned int)strlen(table.entries[row][skill_name_col]) + 1;
    if(!(val->str_name = malloc(name_len))) {
      free(out);
      free_table(&table);
      return NULL; }
    strncpy(val->str_name, table.entries[row][skill_name_col], name_len);
    if(keys == 156) {
      keys = 217; }
    if(keys > size) {
      size *= 2;
      if(!(out = realloc(out, size * sizeof(struct skill_entry_t *)))) {
	free(out);
	free_table(&table);
	return NULL;
      }
      zero_out(out + keys, size / 2 * sizeof(struct skill_entry_t *));
    }
    out[keys] = val;
    keys++;
  }
  free_table(&table);
  if(!keys) {
    free(out);
    return NULL; }
  *out_size = keys;
  return out;
}
