#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tbl.h"
#include "hashtable.h"

/* Makes lots of cstrings for each entry in each string table, stores each
   into a gigantic hashtable and returns the hash.*/
struct hashtable *make_str_hash() {
  unsigned int id_count, entry_count, buffer_size, entry, tbl_offset, pos, key_len, val_len, keys = 0, file_no;
  uint8 *buffer;
  char *key, *val, *key_t, *val_t, filename[1024];
  uint32 tmp;
  FILE *tblfile;
  struct hashtable *out;
  out = create_hashtable(1024, &hash_cstr, &hash_cmp_cstr);
  for(file_no = 0; file_no < 3; file_no++) {
    config(strtbl_file + file_no, filename);
    if(!(tblfile = fopen(filename, "rb"))) {
      write_log("could not open file %s\n", 2, filename);
      return NULL; }
    buffer_size = get_filesize(tblfile);
    if(!(buffer = malloc(buffer_size)) || read_bytes(tblfile, buffer, 0, buffer_size) != buffer_size) {
      fclose(tblfile);
      return NULL; }
    /* The strtbl format isn't too complicated, details can be found at
       http://shadowflare.samods.org/tbld2specs.html */
    id_count = buffer[2] + (buffer[3] << 8);
    entry_count = buffer[4] + (buffer[5] << 8);
    tbl_offset = 21 + id_count * 2;
    for(entry = 0, pos = tbl_offset; entry < entry_count; entry++, pos += 17) {
      tmp = *(uint32 *)(buffer + pos + 7);
      if(tmp > buffer_size) {
	return NULL; }
      key = (char *)buffer + tmp;
      tmp = *(uint32 *)(buffer + pos + 11);
      if(tmp > buffer_size) {
	return NULL; }
      val = (char *)buffer + tmp;
      if(hashtable_search(out, key)) {
	free(hashtable_search(out, key));
	hashtable_remove(out, key); }
      key_len = (unsigned int)strlen(key) + 1;
      if(!(key_t = malloc(key_len))) {
	free(buffer);
	fclose(tblfile);
	return NULL; }
      strncpy(key_t, key, key_len);
      val_len = (unsigned int)strlen(val) + 1;
      if(!(val_t = malloc(val_len))) {
	free(buffer);
	fclose(tblfile);
	return NULL; }
      strncpy(val_t, val, val_len);
      if(!hashtable_insert(out, key_t, val_t)) {
	free(buffer);
	fclose(tblfile);
	return NULL; }
      keys++;
    }
    free(buffer);
    fclose(tblfile);
  }
  if(!keys) {
    hashtable_destroy(out, 1);
    return NULL; }
  return out;
}

/* Looks up a game-official string by its string index found in various tables.
   files used: 'string.tbl', 'expansionstring.tbl', 'patchstring.tbl'
   sample entry: 'ModStr2g' -> 'Increase Maximum Life' */
char *lookup_string_by_name(char *tag) {
  static struct hashtable *lookup = NULL;
  if(!lookup) {
    if(!(lookup = make_str_hash())) {
      write_log("failed to obtain string table\n", 2);
      lookup = (struct hashtable *)1;
      return NULL;
    }
  }
  if(lookup == (struct hashtable *)1) {
    return NULL; }
  else {
    return (char *)hashtable_search(lookup, tag); }
}
