<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
xmlns="http://www.w3.org/1999/xhtml"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:i="http://vidya.dyndns.org/xml/items"
xmlns:p="http://vidya.dyndns.org/xml/properties">
  <xsl:template name="item_listing" match="i:itemlist">
    <xsl:for-each select="i:item">
      <xsl:sort select="i:section"/>
      <xsl:sort select="i:worn" order="descending"/>
      <xsl:sort select="i:quality" order="descending"/>
      <xsl:sort select="i:req_level" order="descending"/>
      <xsl:sort select="i:title"/>
      <xsl:if test="i:location!='socketed into another item'">
	<li class="item_listing">
	  <xsl:attribute name="id">item_listing_<xsl:value-of select="@id"/></xsl:attribute>
	  <xsl:attribute name="onmouseover">delay_show_popup('item<xsl:value-of select="@id"/>', 'item_listing_<xsl:value-of select="@id"/>', 20, 20);</xsl:attribute>
	  <xsl:attribute name="onmouseout">delay_hide_popup();</xsl:attribute>
	  <xsl:if test="i:runeword">
	    <span class="runeword_list">
	      &#34;<xsl:value-of select="i:runeword"/>&#34;, 
	    </span>
	  </xsl:if>
	  <span>
	    <xsl:attribute name="class">
	      <xsl:choose>
		<xsl:when test="i:title/@socketed='yes'">qual_list_socketed</xsl:when>
		<xsl:when test="not(i:quality)">qual_list_normal</xsl:when>
		<xsl:otherwise>qual_list_<xsl:value-of select="i:quality"/></xsl:otherwise>
	      </xsl:choose>
	    </xsl:attribute>
	    <span class="item_list_title">
	      <xsl:value-of select="i:title"/>
	    </span>
	    <xsl:if test="i:tag">
	      <span class="item_list_tag">
		(<xsl:value-of select="i:tag"/>)
	      </span>
	    </xsl:if>
	  </span>
	  <xsl:if test="i:section">
	    <span class="item_list_section">
	      (<xsl:value-of select="i:section"/>)
	    </span>
	  </xsl:if>
	  <span class="item_list_loc">
	    <xsl:choose>
	      <xsl:when test="i:location='stored'">
		(<xsl:value-of select="i:grid_loc"/><!-- at <xsl:value-of select="i:row"/>, <xsl:value-of select="i:col"/>-->)
	      </xsl:when>
	      <xsl:when test="i:location='equipped'">
		(<xsl:value-of select="i:worn"/>)
	      </xsl:when>
	      <xsl:otherwise>
		(<xsl:value-of select="i:location"/>)
	      </xsl:otherwise>
	    </xsl:choose>
	  </span>
	</li>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
  <xsl:template match="i:itemlist">
    <ul class="item_brief_list">
      <xsl:call-template name="item_listing"/>
    </ul>
    <div class="item_list">
      <xsl:for-each select="i:item">
	<xsl:sort select="i:section"/>
	<xsl:sort select="i:worn" order="descending"/>
	<xsl:sort select="i:quality" order="descending"/>
	<xsl:sort select="i:req_level" order="descending"/>
	<xsl:if test="i:location!='socketed into another item'">
	  <div>
	    <xsl:attribute name="class">
	      <xsl:choose>
		<xsl:when test="i:section='mercenary'">item_merc</xsl:when>
		<xsl:when test="i:section='iron golem'">item_ig</xsl:when>
		<xsl:when test="i:section='corpse'">item_corpse</xsl:when>
		<xsl:otherwise>item</xsl:otherwise>
	      </xsl:choose>
	    </xsl:attribute>
	    <xsl:attribute name="onmouseover">preserve_timer('item<xsl:value-of select="@id"/>');</xsl:attribute>
	    <xsl:attribute name="onmouseout">delay_hide_popup();</xsl:attribute>

	    <xsl:attribute name="id">item<xsl:value-of select="@id"/></xsl:attribute>
	    <xsl:if test="i:runeword">
	      <div class="runeword_title">
		<xsl:value-of select="i:runeword"/>
	      </div>
	    </xsl:if>
	    <div>
	      <xsl:attribute name="class">
		<xsl:choose>
		  <xsl:when test="i:title/@socketed='yes'">qual_socketed</xsl:when>
		  <xsl:when test="not(i:quality)">qual_normal</xsl:when>
		  <xsl:otherwise>qual_<xsl:value-of select="i:quality"/></xsl:otherwise>
		</xsl:choose>
	      </xsl:attribute>
	      <div class="item_title">
		<xsl:value-of select="i:title"/>
	      </div>
	      <xsl:if test="i:tag">
		<div class="item_tag">
		  <xsl:value-of select="i:tag"/>
		</div>
	      </xsl:if>
	    </div>
	    <xsl:if test="i:rwtag">
	      <div class="item_rwtag">
		<xsl:value-of select="i:rwtag"/>
	      </div>
	    </xsl:if>
	    <xsl:call-template name="enhancedType">
	      <xsl:with-param name="var" select="i:damage"/>
	      <xsl:with-param name="vartype">dmg</xsl:with-param>
	      <xsl:with-param name="display">Damage: </xsl:with-param>
	    </xsl:call-template>
	    <xsl:call-template name="enhancedType">
	      <xsl:with-param name="var" select="i:defense"/>
	      <xsl:with-param name="vartype">ac</xsl:with-param>
	      <xsl:with-param name="display">Defense: </xsl:with-param>
	    </xsl:call-template>
	    <xsl:call-template name="enhancedType">
	      <xsl:with-param name="var" select="i:block"/>
	      <xsl:with-param name="vartype">block</xsl:with-param>
	      <xsl:with-param name="display">Chance to block: </xsl:with-param>
	    </xsl:call-template>
	    <xsl:if test="i:dura">
	      <div class="item_dura">
		Durability: <xsl:value-of select="i:dura"/>
	      </div>
	    </xsl:if>
	    <xsl:if test="i:quantity">
	      <div class="item_quant">
		Quantity: <xsl:value-of select="i:quantity"/>
	      </div>
	    </xsl:if>
	    <xsl:if test="i:req_str">
	      <div class="item_reqstr">
		Required Strength: <xsl:value-of select="i:req_str"/>
	      </div>
	    </xsl:if>
	    <xsl:if test="i:req_dex">
	      <div class="item_reqdex">
		Required Dexterity: <xsl:value-of select="i:req_dex"/>
	      </div>
	    </xsl:if>
	    <xsl:if test="i:req_level">
	      <div class="item_reqlvl">
		Required Level: <xsl:value-of select="i:req_level"/>
	      </div>
	    </xsl:if>
	    <xsl:if test="i:ilvl">
	      <div class="item_estvalue">
		Item Level: <xsl:value-of select="i:ilvl"/>
	      </div>
	    </xsl:if>
	    <xsl:apply-templates select="p:proplist"/>
	    <xsl:if test="i:sockets">
	      <div class="item_sockets">
		Socketed 
		(<xsl:if test="i:sockets_used"><xsl:value-of select="i:sockets_used"/> / </xsl:if><xsl:value-of select="i:sockets"/>)
	      </div>
	    </xsl:if>
	    <xsl:if test="i:item_set_bonus">
	      <div class="item_setbonus">
		<xsl:apply-templates select="i:item_set_bonus/p:proplist"/>
	      </div>
	    </xsl:if>
	  </div>
	</xsl:if>
      </xsl:for-each>
    </div>
  </xsl:template>
</xsl:stylesheet>
