<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:d2l="http://vidya.dyndns.org/xml/d2list"
xmlns:l="http://vidya.dyndns.org/xml/list">
  <xsl:output method="html" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
  <xsl:template match="l:fullname">
    <div class="listing_fullname">
      <a>
	<xsl:attribute name="href">
	  <xsl:choose>
	    <xsl:when test="l:link"><xsl:value-of select="l:link"/></xsl:when>
	    <xsl:otherwise>javascript:void(0);</xsl:otherwise>
	  </xsl:choose>
	</xsl:attribute>
	<xsl:attribute name="class">
	  <xsl:choose>
	    <xsl:when test="not(l:hardcore) and not(l:dead)">listing_name_notdead</xsl:when>
	    <xsl:when test="l:hardcore and l:dead='yes'">listing_name_dead</xsl:when>
	    <xsl:when test="l:hardcore and not(l:dead)">listing_name_hc</xsl:when>
	    <xsl:otherwise>listing_name</xsl:otherwise>
	  </xsl:choose>
	</xsl:attribute>
	<xsl:if test="l:title"><span class="listing_title"><xsl:value-of select="l:title"/>&#160;</span></xsl:if><span class="listing_shortname"><xsl:value-of select="l:name"/></span>
      </a>
    </div>
    <div class="listing_level">
      Level <xsl:value-of select="l:level"/>
    </div>
    <div class="listing_class">
      <xsl:value-of select="l:class"/>
    </div>
    <div class="listing_spec">
      <xsl:value-of select="l:spec"/>
    </div>
  </xsl:template>
  <xsl:template match="l:list">
    <div class="list">
      <div class="list_title">
	<xsl:value-of select="l:title"/>
      </div>
      <xsl:for-each select="l:listing">
	<div class="listing">
	  <xsl:apply-templates select="l:fullname"/>
	  <div class="listing_val">
	    <xsl:value-of select="l:val"/>
	  </div>
	</div>
      </xsl:for-each>
    </div>
  </xsl:template>
  <xsl:template match="l:hour">
    <xsl:param name="top"/>
    <li class="graph_hour">
      <span class="graph_hour_c">
	<span class="graph_hour_s">
	  <xsl:attribute name="style">
	    height: <xsl:value-of select="(l:hourpercent div $top) * 100"/>%;
	  </xsl:attribute>
	</span>
      </span>
      <span class="graph_hour_p">
	<xsl:value-of select="l:hourpercent"/>%
      </span>
      <span class="graph_hour_t">
	<xsl:value-of select="l:hourval"/>
      </span>
    </li>
  </xsl:template>
  <xsl:template match="l:d2list">
    <div class="header">
      <xsl:if test="l:servername">
	<div class="header_title">
	  Server: <xsl:value-of select="l:servername"/>
	  <xsl:if test="l:season_no">
	    <span class="header_season">
	      - Season <xsl:value-of select="l:season_no"/>
	    </span>
	  </xsl:if>
	</div>
      </xsl:if>
      <div class="header_items">
	<div class="header_soj">
	  Stone of Jordan count: <xsl:value-of select="l:soj"/>
	</div>
	<div class="header_annhi">
	  Annihilus count: <xsl:value-of select="l:annhi"/>
	</div>
	<div class="header_torch">
	  Torch count: <xsl:value-of select="l:torch"/>
	</div>
	<div class="header_std">
	  Standard of Heroes count: <xsl:value-of select="l:std"/>
	</div>
      </div>
      <xsl:if test="l:totalaccts and l:uniqueaccts and l:activeaccts">
	<div class="header_acctinfo">
	  <div class="header_accts">
	    Total accounts: <xsl:value-of select="l:totalaccts"/>
	  </div>
	  <div class="header_uniqueaccts">
	    Unique accounts: <xsl:value-of select="l:uniqueaccts"/>
	  </div>
	  <div class="header_activeaccts">
	    Active accounts: <xsl:value-of select="l:activeaccts"/>
	  </div>
	</div>
      </xsl:if>
      <div class="clear"/>
      <xsl:if test="l:totalgames">
	<div class="header_allgames">
	  Since start:
	  <div class="header_games">
	    Total games: <xsl:value-of select="l:totalgames"/>
	  </div>
	  <div class="header_limgames">
	    Games over 30 min: <xsl:value-of select="l:limgames"/>
	  </div>
	  <div class="header_totlen">
	    Time wasted: <xsl:value-of select="l:totallen"/>
	  </div>
	  <div class="header_avglen">
	    Average length: <xsl:value-of select="l:avglen"/>
	  </div>
	  <div class="header_avgplr">
	    Average players: <xsl:value-of select="l:avgplayers"/>
	  </div>
	</div>
      </xsl:if>
      <xsl:if test="l:recentgames">
	<div class="header_recentgames">
	  Over last five days:
	  <div class="header_recent">
	    Total games: <xsl:value-of select="l:recentgames"/>
	  </div>
	  <div class="header_limgames">
	    Games over 30 min: <xsl:value-of select="l:recentlim"/>
	  </div>
	  <div class="header_totlen">
	    Time wasted: <xsl:value-of select="l:recenttotlen"/>
	  </div>
	  <div class="header_avglen">
	    Average length: <xsl:value-of select="l:recentavglen"/>
	  </div>
	  <div class="header_avgplr">
	    Average players: <xsl:value-of select="l:recentavgplayers"/>
	  </div>
	</div>
      </xsl:if>
    </div>
    <div class="header_charclasses">
      <div class="header_totalchars">
	Total chars: <xsl:value-of select="l:totalchars"/>
      </div>
      <div class="header_avglevel">
	(Average lvl: <xsl:value-of select="l:avglevel"/>)
      </div>
      <div class="header_classtype">
	ama: <span class="header_ama"><xsl:value-of select="l:ama"/></span>
	(avg lvl: <span class="header_ama_lvl"><xsl:value-of select="l:avg_ama"/></span>)
      </div>
      <div class="header_classtype">
	asn: <span class="header_asn"><xsl:value-of select="l:asn"/></span>
	(avg lvl: <span class="header_asn_lvl"><xsl:value-of select="l:avg_asn"/></span>)
      </div>
      <div class="header_classtype">
	bar: <span class="header_bar"><xsl:value-of select="l:bar"/></span>
	(avg lvl: <span class="header_bar_lvl"><xsl:value-of select="l:avg_bar"/></span>)
      </div>
      <div class="header_classtype">
	dru: <span class="header_dru"><xsl:value-of select="l:dru"/></span>
	(avg lvl: <span class="header_dru_lvl"><xsl:value-of select="l:avg_dru"/></span>)
      </div>
      <div class="header_classtype">
	nec: <span class="header_nec"><xsl:value-of select="l:nec"/></span>
	(avg lvl: <span class="header_nec_lvl"><xsl:value-of select="l:avg_nec"/></span>)
      </div>
      <div class="header_classtype">
	pal: <span class="header_pal"><xsl:value-of select="l:pal"/></span>
	(avg lvl: <span class="header_pal_lvl"><xsl:value-of select="l:avg_pal"/></span>)
      </div>
      <div class="header_classtype">
	src: <span class="header_src"><xsl:value-of select="l:src"/></span>
	(avg lvl: <span class="header_src_lvl"><xsl:value-of select="l:avg_src"/></span>)
      </div>
    </div>
    <xsl:if test="l:avgtime">
      <div class="header_avgtime">
	<div class="header_avgtitle">
	  Server activity (total)
	  <xsl:if test="l:timezone">
	    -
	    <span class="header_timezone">
	      All times in <xsl:value-of select="l:timezone"/>
	    </span>
	  </xsl:if>
	</div>
	<ul class="header_graph">
	  <xsl:apply-templates select="l:avgtime/l:hour">
	    <xsl:with-param name="top" select="l:avgtime/l:top"/>
	  </xsl:apply-templates>
	</ul>
      </div>
    </xsl:if>
    <xsl:if test="l:recentavgtime">
      <div class="header_recentavgtime">
	<div class="header_recenttitle">
	  Server activity (last 5 days)
	</div>
	<ul class="header_graph">
	  <xsl:apply-templates select="l:recentavgtime/l:hour">
	    <xsl:with-param name="top" select="l:recentavgtime/l:top"/>
	  </xsl:apply-templates>
	</ul>
      </div>
    </xsl:if>
    <xsl:apply-templates select="l:list"/>
  </xsl:template>
  <xsl:template match="/">
    <html xml:lang="en" lang="en">
    <head>
      <title>Diablo II statistics</title>
      <link rel="stylesheet" type="text/css" href="http://vidya.dyndns.org/xml/list.css"/>
    </head>
    <body>
    <xsl:apply-templates/>
    </body>
    </html>
  </xsl:template>
</xsl:stylesheet>
