#include <math.h>
#include <stdio.h>
#include <string.h>

#include "d2stats.h"
#include "items.h"
#include "tbl.h"
#include "print.h"
#include "xml.h"
#include "tally.h"

/* Prints the 'i:runeword' XML tag. */
void xml_item_runeword(FILE *file, struct item_t *item) {
  char *name, *str;
  if(item->has_runeword == yes) {
    xml_open_tag(file, "i:runeword");
    if((name = lookup_rune_by_id(item->runeword))) {
      str = lookup_string_by_name(name);
      fprintf(file, "%s", str ? str : name); }
    else {
      fprintf(file, "(unknown rune word 0x%02x)", item->runeword); }
    xml_close_tag(file, "i:runeword");
  }
}

/* Prints the 'i:title' XML tag for normal_q items. */
void xml_item_gemmed(FILE *file, struct item_t *item) {
  char *name;
  unsigned int pos;
  struct entry_t *entry;
  enum flag_t is_gemmed = no;
  if(item->sockets) {
    xml_open_attr(file, "i:title", "socketed", "yes");
    if(item->is_personalized == yes) {
      fprintf(file, "%s's ", item->inscription); }
    name = item->tag;
    if(lookup_string_by_name(name)) {
      name = lookup_string_by_name(name); }
    for(pos = 0; pos < item->sockets_used; pos++) {
      if(item->socketed_items[pos] && (entry = lookup_item_by_tag(item->socketed_items[pos]->tag)) && !strncmp(entry->type_tag, "gem", 3)) {
	is_gemmed = yes;
	break;
      }
    }
    if(is_gemmed == yes) {
      fprintf(file, "Gemmed "); }
    fprintf(file, "%s", name);
    xml_close_tag(file, "i:title");
  }
}

/* Prints the 'i:title' XML tag. */
void xml_item_title(FILE *file, struct item_t *item) {
  char *name, *name1, *name2, *str1, *str2;
  if(item->sockets &&
     (item->qual == normal_q ||
      item->qual == low_q ||
      item->qual == high_q)) {
    xml_item_gemmed(file, item);
    return;
  }
  xml_open_tag(file, "i:title");
  if(item->is_ear == yes) {
    fprintf(file, "%s's Ear", (item->ear_info).name); }
  else if(item->name) {
    if(item->is_personalized == yes) {
      fprintf(file, "%s's ", item->inscription); }
    if(item->sockets_used &&
       (item->qual == normal_q ||
	item->qual == low_q ||
	item->qual == high_q)) {
      fprintf(file, "Gemmed "); }
    if(lookup_string_by_name(item->tag)) {
      name = lookup_string_by_name(item->tag); }
    else {
      name = item->name; }
    switch(item->qual) {
    case low_q: fprintf(file, "Low Quality %s", name); break;
    case high_q: fprintf(file, "Superior %s", name); break;
    case magic_q:
      if(item->affix[0]) {
	if((name1 = lookup_magic_pre_by_id(item->affix[0]))) {
	  str1 = lookup_string_by_name(name1);
	  fprintf(file, "%s ", str1 ? str1 : name1); }
	else {
	  fprintf(file, "(unknown prefix 0x%03x) ", item->affix[0]); }
      }
      fprintf(file, "%s", name);
      if(item->affix[1]) {
	if((name2 = lookup_magic_suf_by_id(item->affix[1]))) {
	  str2 = lookup_string_by_name(name2);
	  fprintf(file, " %s", str2 ? str2 : name2); }
	else {
	  fprintf(file, " (unknown suffix 0x%03x)", item->affix[1]); }
      }
      break;
    case rare_q:
    case crafted_q:
      if((name1 = lookup_rare_pre_by_id(item->name1)) &&
	 (name2 = lookup_rare_suf_by_id(item->name2))) {
	str1 = lookup_string_by_name(name1);
	str2 = lookup_string_by_name(name2);
	fprintf(file, "%s %s", str1 ? str1 : name1, str2 ? str2 : name2);
      }
      else {
	fprintf(file, "(unknown rare name 0x%02x 0x%02x)", item->name1, item->name2); }
      break;
    case set_q:
      if(lookup_set_item_by_id(item->suid) && (name1 = lookup_set_item_by_id(item->suid)->name)) {
	str1 = lookup_string_by_name(name1);
	fprintf(file, "%s", str1 ? str1 : name1); }
      else {
	fprintf(file, "(unknown set item 0x%03x)", item->suid); }
      break;
    case unique_q:
      if(item->suid == 0xfff) {
	str1 = lookup_string_by_name(item->tag);
	fprintf(file, "%s", str1 ? str1 : name); }
      else if(lookup_unique_by_id(item->suid) && (name1 = lookup_unique_by_id(item->suid)->name)) {
	str1 = lookup_string_by_name(name1);
	fprintf(file, "%s", str1 ? str1 : name1); }
      else {
	fprintf(file, "(unknown unique item 0x%03x)", item->suid); }
      break;
    default:
      fprintf(file, "%s", name);
    }
  }
  else {
    fprintf(file, "(unknown item tag \"%s\")", item->tag); }
  xml_close_tag(file, "i:title");
}


/* Prints the 'i:tag' XML tag. */
void xml_item_tag(FILE *file, struct item_t *item) {
  char *name, *str;
  if((item->qual == rare_q ||
      item->qual == crafted_q ||
      item->qual == set_q ||
      item->qual == unique_q) && strncmp(item->tag, "std", 4)) {
    xml_open_tag(file, "i:tag");
    name = item->tag;
    str = lookup_string_by_name(name);
    fprintf(file, "%s", str ? str : name);
    xml_close_tag(file, "i:tag");
  }
}

/* Prints the 'i:rwtag' XML tag. */
void xml_item_rwtag(FILE *file, struct item_t *item) {
  char *str, rwtag[64], buf[16];
  unsigned int pos, strpos = 0;
  enum flag_t is_runed = no;
  if((item->qual == low_q || item->qual == normal_q || item->qual == high_q)
     && item->sockets) {
    rwtag[0] = '\0';
    buf[0] = '\0';
    if((str = lookup_string_by_name("RuneQuote"))) {
      sprintf(rwtag, str);
      strpos += (unsigned int)strlen(str); }
    for(pos = 0; pos < item->sockets_used; pos++) {
      if(item->socketed_items[pos] && item->socketed_items[pos]->tag[0] == 'r') {
	is_runed = yes;
	sprintf(buf, "%sL", item->socketed_items[pos]->tag);
	if((str = lookup_string_by_name(buf))) {
	  sprintf(rwtag + strpos, "%s", str);
	  strpos += (unsigned int)strlen(str); }
      }
    }
    if((str = lookup_string_by_name("RuneQuote"))) {
      sprintf(rwtag + strpos, str); }
    if(is_runed == yes) {
      xml_open_tag(file, "i:rwtag");
      fprintf(file, "%s", rwtag);
      xml_close_tag(file, "i:rwtag"); }
  }
}

/* Prints the 'i:quality' XML tag. */    
void xml_item_qual(FILE *file, struct item_t *item) {
  if(item->qual != unknown_q) {
    xml_open_tag(file, "i:quality");
    fprintf(file, "%s", item_qual_to_cstr(item->qual));
    xml_close_tag(file, "i:quality");
  }
}

/* Prints the 'i:damage' XML tag. */
void xml_item_dmg(FILE *file, struct item_t *item) {
  if(item->item_class == weapon_c) {
    if(item->dmg_enhanced) {
      xml_open_attr(file, "i:damage", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:damage"); }
    fprintf(file, "%d to %d", item->min_dmg, item->max_dmg);
    xml_close_tag(file, "i:damage");
  }
}

/* Prints the 'i:defense' XML tag. */
void xml_item_defense(FILE *file, struct item_t *item) {
  if(item->def) {
    if(item->ac_enhanced) {
      xml_open_attr(file, "i:defense", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:defense"); }
    fprintf(file, "%d", item->ac);
    xml_close_tag(file, "i:defense");
  }
}

/* Prints the 'i:block' XML tag. */
void xml_item_block(FILE *file, struct item_t *item) {
  if(item->item_class == shield_c) {
    if(item->block_enhanced) {
      xml_open_attr(file, "i:block", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:block"); }
    fprintf(file, "%d", item->block);
    xml_close_tag(file, "i:block");
  }
}

/* Prints the 'i:dura' XML tag. */
void xml_item_dura(FILE *file, struct item_t *item) {
  if(item->max_dura && item->indestructible == no) {
    if(item->req_enhanced == yes) {
      xml_open_attr(file, "i:dura", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:dura"); }
    fprintf(file, "%d of %d", item->mod_dura, item->mod_max_dura);
    xml_close_tag(file, "i:dura");
  }
}

/* Prints the 'i:quantity' XML tag. */
void xml_item_quant(FILE *file, struct item_t *item) {
  if(item->quantity) {
    xml_open_tag(file, "i:quantity");
    fprintf(file, "%d", item->quantity);
    xml_close_tag(file, "i:quantity");
  }
}

/* Prints the 'i:req_level', 'i:req_str' and 'i:req_dex' XML tags. */
void xml_item_req(FILE *file, struct item_t *item) {
  if(item->req_lvl) {
    xml_open_tag(file, "i:req_level");
    fprintf(file, "%d", item->req_lvl);
    xml_close_tag(file, "i:req_level"); }
  if(item->req_str) {
    if(item->req_enhanced == yes) {
      xml_open_attr(file, "i:req_str", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:req_str"); }
    fprintf(file, "%d", item->req_str);
    xml_close_tag(file, "i:req_str");
  }
  if(item->req_dex) {
    if(item->req_enhanced == yes) {
      xml_open_attr(file, "i:req_dex", "enhanced", "yes"); }
    else {
      xml_open_tag(file, "i:req_dex"); }
    fprintf(file, "%d", item->req_dex);
    xml_close_tag(file, "i:req_dex");
  }
}

/* Prints the 'i:value' XML tag. */
void xml_item_value(FILE *file, struct item_t *item) {
  xml_open_tag(file, "i:value");
  fprintf(file, "%.2f", calc_item_value(item));
  xml_close_tag(file, "i:value");
}

/* Prints the 'i:ilvl' XML tag. */
void xml_item_level(FILE *file, struct item_t *item) {
  if(item->ilvl) {
    xml_open_tag(file, "i:ilvl");
    fprintf(file, "%d", item->ilvl);
    xml_close_tag(file, "i:ilvl");
  }
}

/* Prints a property list, which is a bunch of 'p:prop' XML tags. */
void xml_prop_list(FILE *file, struct prop_t *prop, const unsigned int clvl) {
  unsigned int count, pos, extra;
  struct prop_t *curr, *list[1024];
  if(prop == NULL) {
    return; }
  /* count the properties, make sure prop entries exist, then put pointers
     into a stack-alloc'd array of pointers.*/
  for(curr = prop, count = 0; curr != NULL; curr = curr->next, count++);
  for(curr = prop, pos = 0; curr != NULL; curr = curr->next, pos++) {
    if(!lookup_prop_by_id(curr->id)) {
      return; }
    list[pos] = curr;
  }
  /* then, sort the pointers by priority. */
  sort_prop(list, count);
  for(pos = 0; pos < count; pos++) {
    /* and print each property. if some properties are printed, advance the
       pos into the pointer array. */
    xml_open_tag(file, "p:prop");
    if((extra = print_prop(file, &list[pos], clvl))) {
      xml_close_tag(file, "p:prop");
      if(extra > 1) {
	pos += extra - 1; }
    }
    /* otherwise, mark the prop tag as hidden so that it won't show up. */
    else {
      fprintf(file, "(hidden)");
      xml_close_tag(file, "p:prop");
    }
  }
}

/* Prints the 'i:sockets' and 'i:sockets_used' XML tags. */
void xml_item_socket(FILE *file, struct item_t *item) {
  if(item->sockets) {
    xml_open_tag(file, "i:sockets");
    fprintf(file, "%d", item->sockets);
    xml_close_tag(file, "i:sockets");
  }
  if(item->sockets_used) {
    xml_open_tag(file, "i:sockets_used");
    fprintf(file, "%d", item->sockets_used);
    xml_close_tag(file, "i:sockets_used");
  }
}

/* Prints XML tags that help describe where the item is (on the character). */
void xml_item_loc(FILE *file, struct item_t *item) {
  if(item->section != player) {
    xml_open_tag(file, "i:section");
    fprintf(file, "%s", item_section_to_cstr(item->section));
    xml_close_tag(file, "i:section"); }
  xml_open_tag(file, "i:location");
  fprintf(file, "%s", item_location_to_cstr(item->location));
  xml_close_tag(file, "i:location");
  if(item->location == equipped) {
    xml_open_tag(file, "i:worn");
    fprintf(file, "%s", item_worn_to_cstr(item->worn));
    xml_close_tag(file, "i:worn"); }
  else if(item->location == stored) {
    xml_open_tag(file, "i:grid_loc");
    fprintf(file, "%s", item_grid_loc_to_cstr(item->grid_loc));
    xml_close_tag(file, "i:grid_loc");
    xml_open_tag(file, "i:row");
    fprintf(file, "%d", item->row);
    xml_close_tag(file, "i:row");
    xml_open_tag(file, "i:col");
    fprintf(file, "%d", item->col);
    xml_close_tag(file, "i:col");
  }
  else if(item->location == socketed && item->contained_in) {
    xml_open_tag(file, "i:contained_in");
    fprintf(file, "%d", item->contained_in->id);
    xml_close_tag(file, "i:contained_in");
  }
}

/* Prints the 'i:item' XML tag. */
void xml_item(FILE *file, struct item_t *item, const unsigned int clvl, const uint8 *set_info, const enum char_class_t char_class, const unsigned int id) {
  char buf[8];
  unsigned int set_id, bits_set, pos;
  sprintf(buf, "%d", id); 
  xml_open_tree_attr(file, "i:item", "id", buf);
  xml_item_qual(file, item);
  xml_item_runeword(file, item);
  xml_item_title(file, item);
  xml_item_tag(file, item);
  xml_item_rwtag(file, item);
  xml_item_dmg(file, item);
  xml_item_defense(file, item);
  xml_item_block(file, item);
  xml_item_dura(file, item);
  xml_item_quant(file, item);
  xml_item_req(file, item);
  xml_item_level(file, item);
  if(item->norm_pl) {
    xml_open_tree(file, "p:proplist");
    xml_prop_list(file, item->norm_pl, clvl);
    if(item->is_ethereal) {
      /* add the ethereal property at the end of the proplist, like a normal
	 prop */
      xml_open_tag(file, "p:prop");
      fprintf(file, "Ethereal (Cannot be Repaired)");
      xml_close_tag(file, "p:prop"); }
    xml_close_tree(file, "p:proplist"); }
  if(item->qual == set_q) {
    xml_open_tree(file, "i:item_set_bonus");
    set_id = lookup_set_id_by_suid(item->suid);
    bits_set = count_bits(set_info[set_id]);
    /* determine whether or not each set bonus exists. then, determine whether
       or not the player is wearing enough items to activate it. if it's marked
       disabled it'll show up as red. */
    for(pos = 0; pos < 5; pos++) {
      if(item->has_set_pl[pos]) {
	sprintf(buf, "%d", pos + 2);
	if(bits_set <= pos + 1) {
	  xml_open_tree_2attr(file, "p:proplist", "req", buf, "disabled", "yes"); }
	else {
	  xml_open_tree_attr(file, "p:proplist", "req", buf); }
	xml_prop_list(file, item->set_pl[pos], clvl);
	xml_close_tree(file, "p:proplist");
      }
    }
    xml_close_tree(file, "i:item_set_bonus");
  }
  xml_item_socket(file, item);
  xml_item_loc(file, item);
  xml_close_tree(file, "i:item");
}

/* Prints the 'i:itemlist' XML tag. */
void xml_items(FILE *file, const struct charinfo_t *charinfo) {
  unsigned int item_no;
  xml_open_tree(file, "i:itemlist");
  for(item_no = 0; item_no < charinfo->item_count; item_no++) {
    xml_item(file, charinfo->items[item_no], (charinfo->stats).level, charinfo->set_info, charinfo->char_class, item_no);
  }
  xml_close_tree(file, "i:itemlist");
}
