#include <stdio.h>

#include "d2stats.h"
#include "tbl.h"
#include "tally.h"
#include "print.h"
#include "xml.h"

/* Prints the 'l:fullname' XML tag. */
void xml_list_name(FILE *file, const struct charinfo_t *entry) {
  char *title, buf[64], buf2[1024] = "";
  xml_open_tree(file, "l:fullname");
  if(entry->is_hardcore == yes) {
    xml_open_tag(file, "l:hardcore");
    fprintf(file, "yes");
    xml_close_tag(file, "l:hardcore"); }
  if(entry->is_ladder == yes) {
    xml_open_tag(file, "l:ladder");
    fprintf(file, "yes");
    xml_close_tag(file, "l:ladder"); }
  if(entry->has_died == yes) {
    xml_open_tag(file, "l:dead");
    fprintf(file, "yes");
    xml_close_tag(file, "l:dead"); }
  if((title = char_title_to_cstr(entry->char_title, entry->char_class, entry->is_hardcore)) && title[0] != '\0') {
    xml_open_tag(file, "l:title");
    fprintf(file, "%s", title);
    xml_close_tag(file, "l:title"); }
  xml_open_tag(file, "l:name");
  fprintf(file, "%s", entry->name);
  xml_close_tag(file, "l:name");
  xml_open_tag(file, "l:level");
  fprintf(file, "%d", (entry->stats).level);
  xml_close_tag(file, "l:level");
  xml_open_tag(file, "l:class");
  fprintf(file, "%s", abbr_class_to_cstr(entry->char_class));
  xml_close_tag(file, "l:class");
  get_spec(entry, buf);
  xml_open_tag(file, "l:spec");
  fprintf(file, "%s", buf);
  xml_close_tag(file, "l:spec");
  config(cgi_link, buf2);
  if(buf2[0] != '\0') {
    xml_open_tag(file, "l:link");
    fprintf(file, "%s%s", buf2, entry->name);
    xml_close_tag(file, "l:link"); }
  xml_close_tree(file, "l:fullname");
}

/* the rest of these functions are basically the same, they write lists.
   some are a little different, all are pretty self-explanatory. */
void xml_list_exp(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Experience");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d", (list[pos]->stats).exp);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_ival(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Estimated total item value");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%.2f", (list[pos]->bonus_stats).ival);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_rune(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  char runetag[5], *rune;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Highest rune");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      runetag[0] = 'r';
      runetag[1] = (list[pos]->bonus_stats).high_rune / 10 + '0';
      runetag[2] = (list[pos]->bonus_stats).high_rune % 10 + '0';
      runetag[3] = 'L';
      runetag[4] = '\0';
      rune = lookup_string_by_name(runetag);
      xml_open_tag(file, "l:val");
      if(rune) {
	fprintf(file, "%s", rune); }
      else {
	fprintf(file, "%2d", (list[pos]->bonus_stats).high_rune); }
      if((list[pos]->bonus_stats).high_rune_count > 1) {
	fprintf(file, " x%d", (list[pos]->bonus_stats).high_rune_count); }
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_gold(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total gold");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d", (list[pos]->stats).gold + (list[pos]->stats).gold_stash);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_mf(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total equipped magic find");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d%%", (list[pos]->bonus_stats).mf);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_res(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total equipped bonus resistance");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d%%/%d%%/%d%%/%d%%", (list[pos]->bonus_stats).fire, (list[pos]->bonus_stats).light, (list[pos]->bonus_stats).cold, (list[pos]->bonus_stats).pois);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_skill(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total equipped +skill");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d/%d/%d/%d", (list[pos]->bonus_stats).skill0, (list[pos]->bonus_stats).skill1, (list[pos]->bonus_stats).skill2, (list[pos]->bonus_stats).skill3);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_fcr(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total equipped faster casting rate");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d%%", (list[pos]->bonus_stats).fcr);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_ias(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Total equipped increased attack speed");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d%%", (list[pos]->bonus_stats).ias);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_melee(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Various melee stats (DS, CB, OW)");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d%%/%d%%/%d%%", (list[pos]->bonus_stats).ds, (list[pos]->bonus_stats).cb, (list[pos]->bonus_stats).ow);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_alive(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Players that have not died yet");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d", (list[pos]->stats).exp);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_set(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos, count;
  struct set_entry_t *entry;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Most complete set");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      if((entry = lookup_set_by_sid((list[pos]->bonus_stats).high_set))) {
	count = count_bits(list[pos]->set_info[(list[pos]->bonus_stats).high_set]);
	xml_open_tree(file, "l:listing");
	xml_list_name(file, list[pos]);
	xml_open_tag(file, "l:val");
	fprintf(file, "%s (%d/%d)", entry->name, count, entry->items); 
	xml_close_tag(file, "l:val");
	xml_close_tree(file, "l:listing");
      }
    }
    xml_close_tree(file, "l:list");
  }
}

void xml_list_eardropped(FILE *file, struct charinfo_t **list, const unsigned int entry_count) {
  unsigned int pos;
  if(list[0]) {
    xml_open_tree(file, "l:list");
    xml_open_tag(file, "l:title");
    fprintf(file, "Most dropped ears (more = bad)");
    xml_close_tag(file, "l:title");
    for(pos = 0; pos < entry_count && list[pos]; pos++) {
      xml_open_tree(file, "l:listing");
      xml_list_name(file, list[pos]);
      xml_open_tag(file, "l:val");
      fprintf(file, "%d", (list[pos]->bonus_stats).ears_dropped);
      xml_close_tag(file, "l:val");
      xml_close_tree(file, "l:listing");
    }
    xml_close_tree(file, "l:list");
  }
}

/* Prints 24 XML tags that list time statistics for the server. */
void xml_list_time(FILE *file, const double *times) {
  unsigned int hour;
  double top = 0.0;
  for(hour = 0; hour < 24; hour++) {
    xml_open_tree(file, "l:hour");
    xml_open_tag(file, "l:hourval");
    fprintf(file, "%d", hour ? (hour > 12 ? hour - 12 : hour) : 12);
    fprintf(file, " %s", hour > 11 ? "pm" : "am");
    xml_close_tag(file, "l:hourval");
    xml_open_tag(file, "l:hourpercent");
    fprintf(file, "%.3g", times[hour] * 100);
    xml_close_tag(file, "l:hourpercent");
    xml_close_tree(file, "l:hour");
    if(top < times[hour]) {
      top = times[hour]; }
  }
  /* This tag is used internally by the XSLT parser to scale the bargraph. */
  xml_open_tag(file, "l:top");
  fprintf(file, "%.3g", top * 100);
  xml_close_tag(file, "l:top");
}

/* Prints some 'header' information for the XML list file, describing the
   server a little bit. */
void xml_list_top(FILE *file, struct list_t *list) {
  enum char_class_t pos;
  char class_buf[16], buf[1024], buf2[1024];
  config(server_name, buf);
  if(buf[0] != '\0') {
    xml_open_tag(file, "l:servername");
    fprintf(file, "%s", buf);
    xml_close_tag(file, "l:servername"); }
  config(season_no, buf);
  if(buf[0] != '\0') {
    xml_open_tag(file, "l:season_no");
    fprintf(file, "%s", buf);
    xml_close_tag(file, "l:season_no"); }
  xml_open_tag(file, "l:totalchars");
  fprintf(file, "%d", list->char_count);
  xml_close_tag(file, "l:totalchars");
  xml_open_tag(file, "l:avglevel");
  fprintf(file, "%d", list->avg_level);
  xml_close_tag(file, "l:avglevel");
  if(list->accts || list->unique_accts || list->active_accts) {
    xml_open_tag(file, "l:totalaccts");
    fprintf(file, "%d", list->accts);
    xml_close_tag(file, "l:totalaccts");
    xml_open_tag(file, "l:uniqueaccts");
    fprintf(file, "%d", list->unique_accts);
    xml_close_tag(file, "l:uniqueaccts");
    xml_open_tag(file, "l:activeaccts");
    fprintf(file, "%d", list->active_accts);
    xml_close_tag(file, "l:activeaccts");
  }
  if(list->games) {
    xml_open_tag(file, "l:timezone");
    fprintf(file, "%s", list->timezone);
    xml_close_tag(file, "l:timezone");
    xml_open_tag(file, "l:totalgames");
    fprintf(file, "%d", list->games);
    xml_close_tag(file, "l:totalgames");
    xml_open_tag(file, "l:recentgames");
    fprintf(file, "%d", list->recent);
    xml_close_tag(file, "l:recentgames");
    xml_open_tag(file, "l:avgplayers");
    fprintf(file, "%2.2f", list->avg_players);
    xml_close_tag(file, "l:avgplayers");
    xml_open_tag(file, "l:recentavgplayers");
    fprintf(file, "%2.2f", list->recent_avg_players);
    xml_close_tag(file, "l:recentavgplayers");
    xml_open_tag(file, "l:limgames");
    fprintf(file, "%d", list->games_over_lim);
    xml_close_tag(file, "l:limgames");
    xml_open_tag(file, "l:recentlim");
    fprintf(file, "%d", list->recent_over_lim);
    xml_close_tag(file, "l:recentlim");
    print_time(list->total_game_len, buf2);
    xml_open_tag(file, "l:totallen");
    fprintf(file, "%s", buf2);
    xml_close_tag(file, "l:totallen");
    print_time(list->recent_total_len, buf2);
    xml_open_tag(file, "l:recenttotlen");
    fprintf(file, "%s", buf2);
    xml_close_tag(file, "l:recenttotlen");
    print_time(list->avg_game_len, buf2);
    xml_open_tag(file, "l:avglen");
    fprintf(file, "%s", buf2);
    xml_close_tag(file, "l:avglen");
    print_time(list->recent_avg_len, buf2);
    xml_open_tag(file, "l:recentavglen");
    fprintf(file, "%s", buf2);
    xml_close_tag(file, "l:recentavglen");
    xml_open_tree(file, "l:avgtime");
    xml_list_time(file, list->avg_game_time);
    xml_close_tree(file, "l:avgtime");
    xml_open_tree(file, "l:recentavgtime");
    xml_list_time(file, list->recent_avg_time);
    xml_close_tree(file, "l:recentavgtime");
  }
  /* print out class usage stats for each class. */
  for(pos = ama; pos <= src; pos++) {
    sprintf(class_buf, "l:%s", abbr_class_to_cstr(pos));
    xml_open_tag(file, class_buf);
    fprintf(file, "%d (%2.2f%%)", list->class_count[pos], list->class_count[pos] * 100.0 / list->char_count);
    xml_close_tag(file, class_buf);
    sprintf(class_buf, "l:avg_%s", abbr_class_to_cstr(pos));
    xml_open_tag(file, class_buf);
    fprintf(file, "%d", list->avg_class_level[pos]);
    xml_close_tag(file, class_buf);
  }
  xml_open_tag(file, "l:soj");
  fprintf(file, "%d", list->soj_count);
  xml_close_tag(file, "l:soj");
  xml_open_tag(file, "l:annhi");
  fprintf(file, "%d", list->annhi_count);
  xml_close_tag(file, "l:annhi");
  xml_open_tag(file, "l:torch");
  fprintf(file, "%d", list->torch_count);
  xml_close_tag(file, "l:torch");
  xml_open_tag(file, "l:std");
  fprintf(file, "%d", list->std_count);
  xml_close_tag(file, "l:std");
}

/* Prints the XML list file, minus header / footer. */
void xml_list(FILE *file, struct list_t *list) {
  xml_open_tree(file, "l:d2list");
  xml_list_top(file, list);
  xml_list_exp(file, list->by_exp, 50);
  xml_list_ival(file, list->by_ival, 20);
  xml_list_rune(file, list->by_rune, 20);
  xml_list_gold(file, list->by_gold, 20);
  xml_list_mf(file, list->by_mf, 20);
  xml_list_res(file, list->by_res, 10);
  xml_list_skill(file, list->by_skill, 10);
  xml_list_fcr(file, list->by_fcr, 10);
  xml_list_ias(file, list->by_ias, 10);
  xml_list_melee(file, list->by_melee, 10);
  xml_list_alive(file, list->by_alive, 10);
  xml_list_set(file, list->by_set, 10);
  xml_list_eardropped(file, list->by_ears_dropped, 10);
  xml_close_tree(file, "l:d2list");
}

/* Opens a file handle based on config (or stdout if the config isn't set)
   and prints XML listing info to it. */
void xml_list_print(struct list_t *list) {
  char filename[1024] = "";
  FILE *file;
  get_xml_list_file(list, filename);
  if(filename[0] == '\0') {
    file = stdout; }
  else {
    if(!(file = fopen(filename, "w"))) {
      write_log("could not open file \"%s\" for writing\n", 3, filename);
      return; }
  }
  xml_list_header(file, "d2stats");
  xml_list(file, list);
  xml_list_footer(file, "d2stats");
  if(file != stdout) {
    fclose(file); }
}
